@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,22 +8,41 @@
 #include <string>
 #include <vector>
 
+#include "base/debug/debugging_buildflags.h"
 #include "base/files/file_path.h"
-#include "base/stl_util.h"
 #include "base/strings/strcat.h"
+#include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "build/build_config.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+#if BUILDFLAG(IS_WIN)
+#include <windows.h>
+
+#include <shellapi.h>
+
+#include "base/win/scoped_localalloc.h"
+#endif  // BUILDFLAG(IS_WIN)
+
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+#include "base/run_loop.h"
+#include "base/task/thread_pool.h"
+#include "base/test/bind.h"
+#include "base/test/task_environment.h"
+#endif  // BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+
 namespace base {
 
+#if BUILDFLAG(IS_WIN)
 // To test Windows quoting behavior, we use a string that has some backslashes
 // and quotes.
 // Consider the command-line argument: q\"bs1\bs2\\bs3q\\\"
 // Here it is with C-style escapes.
 static const CommandLine::StringType kTrickyQuoted =
     FILE_PATH_LITERAL("q\\\"bs1\\bs2\\\\bs3q\\\\\\\"");
+#endif
+
 // It should be parsed by Windows as: q"bs1\bs2\\bs3q\"
 // Here that is with C-style escapes.
 static const CommandLine::StringType kTricky =
@@ -49,7 +68,7 @@ TEST(CommandLineTest, CommandLineConstructor) {
       FILE_PATH_LITERAL("--not-a-switch"),
       FILE_PATH_LITERAL("\"in the time of submarines...\""),
       FILE_PATH_LITERAL("unquoted arg-with-space")};
-  CommandLine cl(size(argv), argv);
+  CommandLine cl(std::size(argv), argv);
 
   EXPECT_FALSE(cl.GetCommandLineString().empty());
   EXPECT_FALSE(cl.HasSwitch("cruller"));
@@ -65,7 +84,7 @@ TEST(CommandLineTest, CommandLineConstructor) {
             cl.GetProgram().value());
 
   EXPECT_TRUE(cl.HasSwitch("foo"));
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   EXPECT_TRUE(cl.HasSwitch("bar"));
 #else
   EXPECT_FALSE(cl.HasSwitch("bar"));
@@ -106,16 +125,27 @@ TEST(CommandLineTest, CommandLineConstructor) {
   EXPECT_TRUE(iter == args.end());
 }
 
+TEST(CommandLineTest, CommandLineFromArgvWithoutProgram) {
+  CommandLine::StringVector argv = {FILE_PATH_LITERAL("--switch1"),
+                                    FILE_PATH_LITERAL("--switch2=value2")};
+
+  CommandLine cl = CommandLine::FromArgvWithoutProgram(argv);
+
+  EXPECT_EQ(base::FilePath(), cl.GetProgram());
+  EXPECT_TRUE(cl.HasSwitch("switch1"));
+  EXPECT_EQ("value2", cl.GetSwitchValueASCII("switch2"));
+}
+
 TEST(CommandLineTest, CommandLineFromString) {
-#if defined(OS_WIN)
-  CommandLine cl = CommandLine::FromString(StrCat(
-      {STRING16_LITERAL("program --foo= -bAr  /Spaetzel=pierogi /Baz flim "),
-       STRING16_LITERAL("--other-switches=\"--dog=canine --cat=feline\" "),
-       STRING16_LITERAL("-spaetzle=Crepe   -=loosevalue  FLAN "),
-       STRING16_LITERAL("--input-translation=\"45\"--output-rotation "),
-       STRING16_LITERAL("--quotes="), kTrickyQuoted, STRING16_LITERAL(" "),
-       STRING16_LITERAL("-- -- --not-a-switch "),
-       STRING16_LITERAL("\"in the time of submarines...\"")}));
+#if BUILDFLAG(IS_WIN)
+  CommandLine cl = CommandLine::FromString(
+      L"program --foo= -bAr  /Spaetzel=pierogi /Baz flim "
+      L"--other-switches=\"--dog=canine --cat=feline\" "
+      L"-spaetzle=Crepe   -=loosevalue  FLAN "
+      L"--input-translation=\"45\"--output-rotation "
+      L"--quotes=" +
+      kTrickyQuoted +
+      L" -- -- --not-a-switch \"in the time of submarines...\"");
 
   EXPECT_FALSE(cl.GetCommandLineString().empty());
   EXPECT_FALSE(cl.HasSwitch("cruller"));
@@ -171,8 +201,8 @@ TEST(CommandLineTest, CommandLineFromString) {
 
 // Tests behavior with an empty input string.
 TEST(CommandLineTest, EmptyString) {
-#if defined(OS_WIN)
-  CommandLine cl_from_string = CommandLine::FromString(string16());
+#if BUILDFLAG(IS_WIN)
+  CommandLine cl_from_string = CommandLine::FromString(std::wstring());
   EXPECT_TRUE(cl_from_string.GetCommandLineString().empty());
   EXPECT_TRUE(cl_from_string.GetProgram().empty());
   EXPECT_EQ(1U, cl_from_string.argv().size());
@@ -195,69 +225,52 @@ TEST(CommandLineTest, GetArgumentsString) {
   static const char kSecondArgName[] = "arg2";
   static const char kThirdArgName[] = "arg with space";
   static const char kFourthArgName[] = "nospace";
-  static const char kFifthArgName[] = "%1";
 
   CommandLine cl(CommandLine::NO_PROGRAM);
   cl.AppendSwitchPath(kFirstArgName, FilePath(kPath1));
   cl.AppendSwitchPath(kSecondArgName, FilePath(kPath2));
   cl.AppendArg(kThirdArgName);
   cl.AppendArg(kFourthArgName);
-  cl.AppendArg(kFifthArgName);
 
-#if defined(OS_WIN)
-  CommandLine::StringType expected_first_arg(UTF8ToUTF16(kFirstArgName));
-  CommandLine::StringType expected_second_arg(UTF8ToUTF16(kSecondArgName));
-  CommandLine::StringType expected_third_arg(UTF8ToUTF16(kThirdArgName));
-  CommandLine::StringType expected_fourth_arg(UTF8ToUTF16(kFourthArgName));
-  CommandLine::StringType expected_fifth_arg(UTF8ToUTF16(kFifthArgName));
-#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
+#if BUILDFLAG(IS_WIN)
+  CommandLine::StringType expected_first_arg(UTF8ToWide(kFirstArgName));
+  CommandLine::StringType expected_second_arg(UTF8ToWide(kSecondArgName));
+  CommandLine::StringType expected_third_arg(UTF8ToWide(kThirdArgName));
+  CommandLine::StringType expected_fourth_arg(UTF8ToWide(kFourthArgName));
+#elif BUILDFLAG(IS_POSIX) || BUILDFLAG(IS_FUCHSIA)
   CommandLine::StringType expected_first_arg(kFirstArgName);
   CommandLine::StringType expected_second_arg(kSecondArgName);
   CommandLine::StringType expected_third_arg(kThirdArgName);
   CommandLine::StringType expected_fourth_arg(kFourthArgName);
-  CommandLine::StringType expected_fifth_arg(kFifthArgName);
 #endif
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 #define QUOTE_ON_WIN FILE_PATH_LITERAL("\"")
 #else
 #define QUOTE_ON_WIN FILE_PATH_LITERAL("")
-#endif  // OS_WIN
+#endif  // BUILDFLAG(IS_WIN)
 
   CommandLine::StringType expected_str;
   expected_str.append(FILE_PATH_LITERAL("--"))
-              .append(expected_first_arg)
-              .append(FILE_PATH_LITERAL("="))
-              .append(QUOTE_ON_WIN)
-              .append(kPath1)
-              .append(QUOTE_ON_WIN)
-              .append(FILE_PATH_LITERAL(" "))
-              .append(FILE_PATH_LITERAL("--"))
-              .append(expected_second_arg)
-              .append(FILE_PATH_LITERAL("="))
-              .append(QUOTE_ON_WIN)
-              .append(kPath2)
-              .append(QUOTE_ON_WIN)
-              .append(FILE_PATH_LITERAL(" "))
-              .append(QUOTE_ON_WIN)
-              .append(expected_third_arg)
-              .append(QUOTE_ON_WIN)
-              .append(FILE_PATH_LITERAL(" "))
-              .append(expected_fourth_arg)
-              .append(FILE_PATH_LITERAL(" "));
-
-  CommandLine::StringType expected_str_no_quote_placeholders(expected_str);
-  expected_str_no_quote_placeholders.append(expected_fifth_arg);
-  EXPECT_EQ(expected_str_no_quote_placeholders, cl.GetArgumentsString());
-
-#if defined(OS_WIN)
-  CommandLine::StringType expected_str_quote_placeholders(expected_str);
-  expected_str_quote_placeholders.append(QUOTE_ON_WIN)
-                                 .append(expected_fifth_arg)
-                                 .append(QUOTE_ON_WIN);
-  EXPECT_EQ(expected_str_quote_placeholders,
-            cl.GetArgumentsStringWithPlaceholders());
-#endif
+      .append(expected_first_arg)
+      .append(FILE_PATH_LITERAL("="))
+      .append(QUOTE_ON_WIN)
+      .append(kPath1)
+      .append(QUOTE_ON_WIN)
+      .append(FILE_PATH_LITERAL(" "))
+      .append(FILE_PATH_LITERAL("--"))
+      .append(expected_second_arg)
+      .append(FILE_PATH_LITERAL("="))
+      .append(QUOTE_ON_WIN)
+      .append(kPath2)
+      .append(QUOTE_ON_WIN)
+      .append(FILE_PATH_LITERAL(" "))
+      .append(QUOTE_ON_WIN)
+      .append(expected_third_arg)
+      .append(QUOTE_ON_WIN)
+      .append(FILE_PATH_LITERAL(" "))
+      .append(expected_fourth_arg);
+  EXPECT_EQ(expected_str, cl.GetArgumentsString());
 }
 
 // Test methods for appending switches to a command line.
@@ -291,43 +304,156 @@ TEST(CommandLineTest, AppendSwitches) {
   EXPECT_TRUE(cl.HasSwitch(switch5));
   EXPECT_EQ(value5, cl.GetSwitchValueNative(switch5));
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   EXPECT_EQ(
-      StrCat({STRING16_LITERAL("Program "), STRING16_LITERAL("--switch1 "),
-              STRING16_LITERAL("--switch2=value "),
-              STRING16_LITERAL("--switch3=\"a value with spaces\" "),
-              STRING16_LITERAL("--switch4=\"\\\"a value with quotes\\\"\" "),
-              // Even though the switches are unique, appending can add
-              // repeat switches to argv.
-              STRING16_LITERAL("--quotes=\"\\\"a value with quotes\\\"\" "),
-              STRING16_LITERAL("--quotes=\""), kTrickyQuoted,
-              STRING16_LITERAL("\"")}),
+      L"Program "
+      L"--switch1 "
+      L"--switch2=value "
+      L"--switch3=\"a value with spaces\" "
+      L"--switch4=\"\\\"a value with quotes\\\"\" "
+      // Even though the switches are unique, appending can add repeat
+      // switches to argv.
+      L"--quotes=\"\\\"a value with quotes\\\"\" "
+      L"--quotes=\"" +
+          kTrickyQuoted + L"\"",
       cl.GetCommandLineString());
 #endif
 }
 
 TEST(CommandLineTest, AppendSwitchesDashDash) {
- const CommandLine::CharType* raw_argv[] = { FILE_PATH_LITERAL("prog"),
-                                             FILE_PATH_LITERAL("--"),
-                                             FILE_PATH_LITERAL("--arg1") };
- CommandLine cl(size(raw_argv), raw_argv);
+  const CommandLine::CharType* const raw_argv[] = {FILE_PATH_LITERAL("prog"),
+                                                   FILE_PATH_LITERAL("--"),
+                                                   FILE_PATH_LITERAL("--arg1")};
+  CommandLine cl(std::size(raw_argv), raw_argv);
 
- cl.AppendSwitch("switch1");
- cl.AppendSwitchASCII("switch2", "foo");
+  cl.AppendSwitch("switch1");
+  cl.AppendSwitchASCII("switch2", "foo");
 
- cl.AppendArg("--arg2");
+  cl.AppendArg("--arg2");
 
- EXPECT_EQ(FILE_PATH_LITERAL("prog --switch1 --switch2=foo -- --arg1 --arg2"),
-           cl.GetCommandLineString());
- CommandLine::StringVector cl_argv = cl.argv();
- EXPECT_EQ(FILE_PATH_LITERAL("prog"), cl_argv[0]);
- EXPECT_EQ(FILE_PATH_LITERAL("--switch1"), cl_argv[1]);
- EXPECT_EQ(FILE_PATH_LITERAL("--switch2=foo"), cl_argv[2]);
- EXPECT_EQ(FILE_PATH_LITERAL("--"), cl_argv[3]);
- EXPECT_EQ(FILE_PATH_LITERAL("--arg1"), cl_argv[4]);
- EXPECT_EQ(FILE_PATH_LITERAL("--arg2"), cl_argv[5]);
+  EXPECT_EQ(FILE_PATH_LITERAL("prog --switch1 --switch2=foo -- --arg1 --arg2"),
+            cl.GetCommandLineString());
+  CommandLine::StringVector cl_argv = cl.argv();
+  EXPECT_EQ(FILE_PATH_LITERAL("prog"), cl_argv[0]);
+  EXPECT_EQ(FILE_PATH_LITERAL("--switch1"), cl_argv[1]);
+  EXPECT_EQ(FILE_PATH_LITERAL("--switch2=foo"), cl_argv[2]);
+  EXPECT_EQ(FILE_PATH_LITERAL("--"), cl_argv[3]);
+  EXPECT_EQ(FILE_PATH_LITERAL("--arg1"), cl_argv[4]);
+  EXPECT_EQ(FILE_PATH_LITERAL("--arg2"), cl_argv[5]);
 }
 
+#if BUILDFLAG(IS_WIN)
+struct CommandLineQuoteTestCase {
+  const wchar_t* const input_arg;
+  const wchar_t* const expected_output_arg;
+};
+
+class CommandLineQuoteTest
+    : public ::testing::TestWithParam<CommandLineQuoteTestCase> {};
+
+INSTANTIATE_TEST_SUITE_P(
+    CommandLineQuoteTestCases,
+    CommandLineQuoteTest,
+    ::testing::ValuesIn(std::vector<CommandLineQuoteTestCase>{
+        {L"", L""},
+        {L"abc = xyz", LR"("abc = xyz")"},
+        {LR"(C:\AppData\Local\setup.exe)", LR"("C:\AppData\Local\setup.exe")"},
+        {LR"(C:\Program Files\setup.exe)", LR"("C:\Program Files\setup.exe")"},
+        {LR"("C:\Program Files\setup.exe")",
+         LR"("\"C:\Program Files\setup.exe\"")"},
+    }));
+
+TEST_P(CommandLineQuoteTest, TestCases) {
+  EXPECT_EQ(CommandLine::QuoteForCommandLineToArgvW(GetParam().input_arg),
+            GetParam().expected_output_arg);
+}
+
+struct CommandLineQuoteAfterTestCase {
+  const std::vector<std::wstring> input_args;
+  const wchar_t* const expected_output;
+};
+
+class CommandLineQuoteAfterTest
+    : public ::testing::TestWithParam<CommandLineQuoteAfterTestCase> {};
+
+INSTANTIATE_TEST_SUITE_P(
+    CommandLineQuoteAfterTestCases,
+    CommandLineQuoteAfterTest,
+    ::testing::ValuesIn(std::vector<CommandLineQuoteAfterTestCase>{
+        {{L"abc=1"}, L"abc=1"},
+        {{L"abc=1", L"xyz=2"}, L"abc=1 xyz=2"},
+        {{L"abc=1", L"xyz=2", L"q"}, L"abc=1 xyz=2 q"},
+        {{L" abc=1  ", L"  xyz=2", L"q "}, L"abc=1 xyz=2 q"},
+        {{LR"("abc = 1")"}, LR"("abc = 1")"},
+        {{LR"(abc" = "1)", L"xyz=2"}, LR"("abc = 1" xyz=2)"},
+        {{LR"(abc" = "1)"}, LR"("abc = 1")"},
+        {{LR"(\\)", LR"(\\\")"}, LR"("\\\\" "\\\"")"},
+    }));
+
+TEST_P(CommandLineQuoteAfterTest, TestCases) {
+  std::wstring input_command_line =
+      base::StrCat({LR"(c:\test\process.exe )",
+                    base::JoinString(GetParam().input_args, L" ")});
+  int num_args = 0;
+  base::win::ScopedLocalAllocTyped<wchar_t*> argv(
+      ::CommandLineToArgvW(&input_command_line[0], &num_args));
+  ASSERT_EQ(num_args - 1U, GetParam().input_args.size());
+
+  std::wstring recreated_command_line;
+  for (int i = 1; i < num_args; ++i) {
+    recreated_command_line.append(
+        CommandLine::QuoteForCommandLineToArgvW(argv.get()[i]));
+
+    if (i + 1 < num_args) {
+      recreated_command_line.push_back(L' ');
+    }
+  }
+
+  EXPECT_EQ(recreated_command_line, GetParam().expected_output);
+}
+
+TEST(CommandLineTest, GetCommandLineStringForShell) {
+  CommandLine cl = CommandLine::FromString(
+      FILE_PATH_LITERAL("program --switch /switch2 --"));
+  EXPECT_EQ(
+      cl.GetCommandLineStringForShell(),
+      FILE_PATH_LITERAL("program --switch /switch2 -- --single-argument %1"));
+}
+
+TEST(CommandLineTest, GetCommandLineStringWithUnsafeInsertSequences) {
+  CommandLine cl(FilePath(FILE_PATH_LITERAL("program")));
+  cl.AppendSwitchASCII("switch", "%1");
+  cl.AppendSwitch("%2");
+  cl.AppendArg("%3");
+  EXPECT_EQ(FILE_PATH_LITERAL("program --switch=%1 --%2 %3"),
+            cl.GetCommandLineStringWithUnsafeInsertSequences());
+}
+
+TEST(CommandLineTest, HasSingleArgument) {
+  CommandLine cl(FilePath(FILE_PATH_LITERAL("Program")));
+  cl.AppendSwitchASCII("switch2", "foo");
+  EXPECT_FALSE(cl.HasSingleArgumentSwitch());
+  CommandLine cl_for_shell(
+      CommandLine::FromString(cl.GetCommandLineStringForShell()));
+  EXPECT_TRUE(cl_for_shell.HasSingleArgumentSwitch());
+}
+
+// Test that creating a new command line from the string version of a single
+// argument command line maintains the single argument switch, and the
+// argument.
+TEST(CommandLineTest, MaintainSingleArgument) {
+  // Putting a space in the file name will force escaping of the argument.
+  static const CommandLine::StringType kCommandLine =
+      FILE_PATH_LITERAL("program --switch --single-argument foo bar.html");
+  CommandLine cl = CommandLine::FromString(kCommandLine);
+  CommandLine cl_for_shell = CommandLine::FromString(cl.GetCommandLineString());
+  EXPECT_TRUE(cl_for_shell.HasSingleArgumentSwitch());
+  // Verify that we command line survives the round trip with an escaped arg.
+  EXPECT_EQ(kCommandLine, cl_for_shell.GetCommandLineString());
+}
+
+#endif  // BUILDFLAG(IS_WIN)
+
 // Tests that when AppendArguments is called that the program is set correctly
 // on the target CommandLine object and the switches from the source
 // CommandLine are added to the target.
@@ -352,18 +478,18 @@ TEST(CommandLineTest, AppendArguments) {
   EXPECT_TRUE(c1.HasSwitch("switch2"));
 }
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 // Make sure that the command line string program paths are quoted as necessary.
 // This only makes sense on Windows and the test is basically here to guard
 // against regressions.
 TEST(CommandLineTest, ProgramQuotes) {
   // Check that quotes are not added for paths without spaces.
-  const FilePath kProgram(STRING16_LITERAL("Program"));
+  const FilePath kProgram(L"Program");
   CommandLine cl_program(kProgram);
   EXPECT_EQ(kProgram.value(), cl_program.GetProgram().value());
   EXPECT_EQ(kProgram.value(), cl_program.GetCommandLineString());
 
-  const FilePath kProgramPath(STRING16_LITERAL("Program Path"));
+  const FilePath kProgramPath(L"Program Path");
 
   // Check that quotes are not returned from GetProgram().
   CommandLine cl_program_path(kProgramPath);
@@ -371,14 +497,7 @@ TEST(CommandLineTest, ProgramQuotes) {
 
   // Check that quotes are added to command line string paths containing spaces.
   CommandLine::StringType cmd_string(cl_program_path.GetCommandLineString());
-  EXPECT_EQ(STRING16_LITERAL("\"Program Path\""), cmd_string);
-
-  // Check the optional quoting of placeholders in programs.
-  CommandLine cl_quote_placeholder(FilePath(STRING16_LITERAL("%1")));
-  EXPECT_EQ(STRING16_LITERAL("%1"),
-            cl_quote_placeholder.GetCommandLineString());
-  EXPECT_EQ(STRING16_LITERAL("\"%1\""),
-            cl_quote_placeholder.GetCommandLineStringWithPlaceholders());
+  EXPECT_EQ(L"\"Program Path\"", cmd_string);
 }
 #endif
 
@@ -395,8 +514,7 @@ TEST(CommandLineTest, Init) {
 
 // Test that copies of CommandLine have a valid StringPiece map.
 TEST(CommandLineTest, Copy) {
-  std::unique_ptr<CommandLine> initial(
-      new CommandLine(CommandLine::NO_PROGRAM));
+  auto initial = std::make_unique<CommandLine>(CommandLine::NO_PROGRAM);
   initial->AppendSwitch("a");
   initial->AppendSwitch("bbbbbbbbbbbbbbb");
   initial->AppendSwitch("c");
@@ -410,6 +528,29 @@ TEST(CommandLineTest, Copy) {
     EXPECT_TRUE(assigned.HasSwitch(pair.first));
 }
 
+TEST(CommandLineTest, CopySwitches) {
+  CommandLine source(CommandLine::NO_PROGRAM);
+  source.AppendSwitch("a");
+  source.AppendSwitch("bbbb");
+  source.AppendSwitch("c");
+  EXPECT_THAT(source.argv(), testing::ElementsAre(FILE_PATH_LITERAL(""),
+                                                  FILE_PATH_LITERAL("--a"),
+                                                  FILE_PATH_LITERAL("--bbbb"),
+                                                  FILE_PATH_LITERAL("--c")));
+
+  CommandLine cl(CommandLine::NO_PROGRAM);
+  EXPECT_THAT(cl.argv(), testing::ElementsAre(FILE_PATH_LITERAL("")));
+
+  cl.CopySwitchesFrom(source, {});
+  EXPECT_THAT(cl.argv(), testing::ElementsAre(FILE_PATH_LITERAL("")));
+
+  static const char* const kSwitchesToCopy[] = {"a", "nosuch", "c"};
+  cl.CopySwitchesFrom(source, kSwitchesToCopy);
+  EXPECT_THAT(cl.argv(), testing::ElementsAre(FILE_PATH_LITERAL(""),
+                                              FILE_PATH_LITERAL("--a"),
+                                              FILE_PATH_LITERAL("--c")));
+}
+
 TEST(CommandLineTest, PrependSimpleWrapper) {
   CommandLine cl(FilePath(FILE_PATH_LITERAL("Program")));
   cl.AppendSwitch("a");
@@ -495,6 +636,28 @@ TEST(CommandLineTest, RemoveSwitchWithValue) {
                                               FILE_PATH_LITERAL("--switch1")));
 }
 
+TEST(CommandLineTest, RemoveSwitchDropsMultipleSameSwitches) {
+  const std::string switch1 = "switch1";
+  const std::string value2 = "value2";
+
+  CommandLine cl(FilePath(FILE_PATH_LITERAL("Program")));
+
+  cl.AppendSwitch(switch1);
+  cl.AppendSwitchASCII(switch1, value2);
+
+  EXPECT_TRUE(cl.HasSwitch(switch1));
+  EXPECT_EQ(value2, cl.GetSwitchValueASCII(switch1));
+  EXPECT_THAT(cl.argv(),
+              testing::ElementsAre(FILE_PATH_LITERAL("Program"),
+                                   FILE_PATH_LITERAL("--switch1"),
+                                   FILE_PATH_LITERAL("--switch1=value2")));
+
+  cl.RemoveSwitch(switch1);
+
+  EXPECT_FALSE(cl.HasSwitch(switch1));
+  EXPECT_THAT(cl.argv(), testing::ElementsAre(FILE_PATH_LITERAL("Program")));
+}
+
 TEST(CommandLineTest, AppendAndRemoveSwitchWithDefaultPrefix) {
   CommandLine cl(FilePath(FILE_PATH_LITERAL("Program")));
 
@@ -547,7 +710,7 @@ TEST(CommandLineTest, MultipleSameSwitch) {
       FILE_PATH_LITERAL("-baz"),
       FILE_PATH_LITERAL("--foo=two")  // --foo second time
   };
-  CommandLine cl(size(argv), argv);
+  CommandLine cl(std::size(argv), argv);
 
   EXPECT_TRUE(cl.HasSwitch("foo"));
   EXPECT_TRUE(cl.HasSwitch("baz"));
@@ -555,4 +718,96 @@ TEST(CommandLineTest, MultipleSameSwitch) {
   EXPECT_EQ("two", cl.GetSwitchValueASCII("foo"));
 }
 
-} // namespace base
+// Helper class for the next test case
+class MergeDuplicateFoosSemicolon : public DuplicateSwitchHandler {
+ public:
+  ~MergeDuplicateFoosSemicolon() override;
+
+  void ResolveDuplicate(base::StringPiece key,
+                        CommandLine::StringPieceType new_value,
+                        CommandLine::StringType& out_value) override;
+};
+
+MergeDuplicateFoosSemicolon::~MergeDuplicateFoosSemicolon() = default;
+
+void MergeDuplicateFoosSemicolon::ResolveDuplicate(
+    base::StringPiece key,
+    CommandLine::StringPieceType new_value,
+    CommandLine::StringType& out_value) {
+  if (key != "mergeable-foo") {
+    out_value = CommandLine::StringType(new_value);
+    return;
+  }
+  if (!out_value.empty()) {
+#if BUILDFLAG(IS_WIN)
+    StrAppend(&out_value, {L";"});
+#else
+    StrAppend(&out_value, {";"});
+#endif
+  }
+  StrAppend(&out_value, {new_value});
+}
+
+// This flag is an exception to the rule that the second duplicate flag wins
+// Not thread safe
+TEST(CommandLineTest, MultipleFilterFileSwitch) {
+  const CommandLine::CharType* const argv[] = {
+      FILE_PATH_LITERAL("program"),
+      FILE_PATH_LITERAL("--mergeable-foo=one"),  // --first time
+      FILE_PATH_LITERAL("-baz"),
+      FILE_PATH_LITERAL("--mergeable-foo=two")  // --second time
+  };
+  CommandLine::SetDuplicateSwitchHandler(
+      std::make_unique<MergeDuplicateFoosSemicolon>());
+
+  CommandLine cl(std::size(argv), argv);
+
+  EXPECT_TRUE(cl.HasSwitch("mergeable-foo"));
+  EXPECT_TRUE(cl.HasSwitch("baz"));
+
+  EXPECT_EQ("one;two", cl.GetSwitchValueASCII("mergeable-foo"));
+  CommandLine::SetDuplicateSwitchHandler(nullptr);
+}
+
+#if BUILDFLAG(IS_WIN)
+TEST(CommandLineTest, ParseAsSingleArgument) {
+  CommandLine cl = CommandLine::FromString(
+      FILE_PATH_LITERAL("program --switch_before arg_before "
+                        "--single-argument arg with spaces \"and quotes\" \""));
+
+  EXPECT_FALSE(cl.GetCommandLineString().empty());
+  EXPECT_EQ(FilePath(FILE_PATH_LITERAL("program")), cl.GetProgram());
+  EXPECT_TRUE(cl.HasSwitch("switch_before"));
+  EXPECT_EQ(cl.GetArgs(), CommandLine::StringVector({FILE_PATH_LITERAL(
+                              "arg with spaces \"and quotes\" \"")}));
+
+  CommandLine cl_without_arg =
+      CommandLine::FromString(FILE_PATH_LITERAL("program --single-argument "));
+
+  EXPECT_FALSE(cl_without_arg.GetCommandLineString().empty());
+  EXPECT_EQ(FilePath(FILE_PATH_LITERAL("program")),
+            cl_without_arg.GetProgram());
+  EXPECT_TRUE(cl_without_arg.GetArgs().empty());
+}
+#endif  // BUILDFLAG(IS_WIN)
+
+#if BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+TEST(CommandLineDeathTest, ThreadChecks) {
+  test::TaskEnvironment task_environment;
+  RunLoop run_loop;
+  EXPECT_DEATH_IF_SUPPORTED(
+      {
+        ThreadPool::PostTask(FROM_HERE, BindLambdaForTesting([&run_loop]() {
+                               auto* command_line =
+                                   CommandLine::ForCurrentProcess();
+                               command_line->AppendSwitch("test");
+                               run_loop.Quit();
+                             }));
+
+        run_loop.Run();
+      },
+      "");
+}
+#endif  // BUILDFLAG(ENABLE_COMMANDLINE_SEQUENCE_CHECKS)
+
+}  // namespace base
