@@ -1,4 +1,4 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,9 +7,12 @@
 #include <limits.h>
 #include <stddef.h>
 
+#include <utility>
+
+#include "base/containers/span.h"
 #include "base/logging.h"
+#include "base/notreached.h"
 #include "base/rand_util.h"
-#include "base/stl_util.h"
 #include "base/threading/scoped_blocking_call.h"
 #include "base/win/scoped_handle.h"
 
@@ -22,7 +25,7 @@ namespace {
 // in sandboxed scenarios as we might have by-name policies that allow pipe
 // creation. Also keep the secure random number generation.
 const wchar_t kPipeNameFormat[] = L"\\\\.\\pipe\\chrome.sync.%u.%u.%lu";
-const size_t kPipePathMax = base::size(kPipeNameFormat) + (3 * 10) + 1;
+const size_t kPipePathMax = std::size(kPipeNameFormat) + (3 * 10) + 1;
 
 // To avoid users sending negative message lengths to Send/Receive
 // we clamp message lengths, which are size_t, to no more than INT_MAX.
@@ -32,10 +35,12 @@ const int kOutBufferSize = 4096;
 const int kInBufferSize = 4096;
 const int kDefaultTimeoutMilliSeconds = 1000;
 
-bool CreatePairImpl(HANDLE* socket_a, HANDLE* socket_b, bool overlapped) {
+bool CreatePairImpl(ScopedHandle* socket_a,
+                    ScopedHandle* socket_b,
+                    bool overlapped) {
   DCHECK_NE(socket_a, socket_b);
-  DCHECK_EQ(*socket_a, SyncSocket::kInvalidHandle);
-  DCHECK_EQ(*socket_b, SyncSocket::kInvalidHandle);
+  DCHECK(!socket_a->is_valid());
+  DCHECK(!socket_b->is_valid());
 
   wchar_t name[kPipePathMax];
   ScopedHandle handle_a;
@@ -62,10 +67,9 @@ bool CreatePairImpl(HANDLE* socket_a, HANDLE* socket_b, bool overlapped) {
         kInBufferSize,
         kDefaultTimeoutMilliSeconds,
         NULL));
-  } while (!handle_a.IsValid() &&
-           (GetLastError() == ERROR_PIPE_BUSY));
+  } while (!handle_a.is_valid() && (GetLastError() == ERROR_PIPE_BUSY));
 
-  if (!handle_a.IsValid()) {
+  if (!handle_a.is_valid()) {
     NOTREACHED();
     return false;
   }
@@ -84,12 +88,12 @@ bool CreatePairImpl(HANDLE* socket_a, HANDLE* socket_b, bool overlapped) {
                                     OPEN_EXISTING,  // opens existing pipe.
                                     flags,
                                     NULL));     // no template file.
-  if (!handle_b.IsValid()) {
+  if (!handle_b.is_valid()) {
     DPLOG(ERROR) << "CreateFileW failed";
     return false;
   }
 
-  if (!ConnectNamedPipe(handle_a.Get(), NULL)) {
+  if (!ConnectNamedPipe(handle_a.get(), NULL)) {
     DWORD error = GetLastError();
     if (error != ERROR_PIPE_CONNECTED) {
       DPLOG(ERROR) << "ConnectNamedPipe failed";
@@ -97,8 +101,8 @@ bool CreatePairImpl(HANDLE* socket_a, HANDLE* socket_b, bool overlapped) {
     }
   }
 
-  *socket_a = handle_a.Take();
-  *socket_b = handle_b.Take();
+  *socket_a = std::move(handle_a);
+  *socket_b = std::move(handle_b);
 
   return true;
 }
@@ -114,28 +118,26 @@ DWORD GetNextChunkSize(size_t current_pos, size_t max_size) {
 // overlapped fashion and waits for IO completion.  The function also waits
 // on an event that can be used to cancel the operation.  If the operation
 // is cancelled, the function returns and closes the relevant socket object.
-template <typename BufferType, typename Function>
+template <typename DataType, typename Function>
 size_t CancelableFileOperation(Function operation,
                                HANDLE file,
-                               BufferType* buffer,
-                               size_t length,
+                               span<DataType> buffer,
                                WaitableEvent* io_event,
                                WaitableEvent* cancel_event,
                                CancelableSyncSocket* socket,
                                DWORD timeout_in_ms) {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
   // The buffer must be byte size or the length check won't make much sense.
-  static_assert(sizeof(buffer[0]) == sizeof(char), "incorrect buffer type");
-  DCHECK_GT(length, 0u);
-  DCHECK_LE(length, kMaxMessageLength);
-  DCHECK_NE(file, SyncSocket::kInvalidHandle);
+  static_assert(sizeof(DataType) == 1u, "incorrect buffer type");
+  CHECK(!buffer.empty());
+  CHECK_LE(buffer.size(), kMaxMessageLength);
+  CHECK_NE(file, SyncSocket::kInvalidHandle);
 
   // Track the finish time so we can calculate the timeout as data is read.
   TimeTicks current_time, finish_time;
   if (timeout_in_ms != INFINITE) {
     current_time = TimeTicks::Now();
-    finish_time =
-        current_time + base::TimeDelta::FromMilliseconds(timeout_in_ms);
+    finish_time = current_time + base::Milliseconds(timeout_in_ms);
   }
 
   size_t count = 0;
@@ -144,17 +146,22 @@ size_t CancelableFileOperation(Function operation,
     OVERLAPPED ol = { 0 };
     ol.hEvent = io_event->handle();
 
-    const DWORD chunk = GetNextChunkSize(count, length);
+    const DWORD chunk_size = GetNextChunkSize(count, buffer.size());
     // This is either the ReadFile or WriteFile call depending on whether
     // we're receiving or sending data.
     DWORD len = 0;
-    const BOOL operation_ok = operation(
-        file, static_cast<BufferType*>(buffer) + count, chunk, &len, &ol);
+    auto operation_buffer = buffer.subspan(count, chunk_size);
+    // SAFETY: The below static_cast is in range for DWORD because
+    // `operation_buffer` is constructed with a DWORD length above from
+    // `chunk_size`.
+    const BOOL operation_ok =
+        operation(file, operation_buffer.data(),
+                  static_cast<DWORD>(operation_buffer.size()), &len, &ol);
     if (!operation_ok) {
       if (::GetLastError() == ERROR_IO_PENDING) {
         HANDLE events[] = { io_event->handle(), cancel_event->handle() };
-        const int wait_result = WaitForMultipleObjects(
-            base::size(events), events, FALSE,
+        const DWORD wait_result = WaitForMultipleObjects(
+            std::size(events), events, FALSE,
             timeout_in_ms == INFINITE
                 ? timeout_in_ms
                 : static_cast<DWORD>(
@@ -188,14 +195,16 @@ size_t CancelableFileOperation(Function operation,
     count += len;
 
     // Quit the operation if we can't write/read anymore.
-    if (len != chunk)
+    if (len != chunk_size) {
       break;
+    }
 
     // Since TimeTicks::Now() is expensive, only bother updating the time if we
     // have more work to do.
-    if (timeout_in_ms != INFINITE && count < length)
+    if (timeout_in_ms != INFINITE && count < buffer.size()) {
       current_time = base::TimeTicks::Now();
-  } while (count < length &&
+    }
+  } while (count < buffer.size() &&
            (timeout_in_ms == INFINITE || current_time < finish_time));
 
   return count;
@@ -203,58 +212,29 @@ size_t CancelableFileOperation(Function operation,
 
 }  // namespace
 
-#if defined(COMPONENT_BUILD)
-const SyncSocket::Handle SyncSocket::kInvalidHandle = INVALID_HANDLE_VALUE;
-#endif
-
-SyncSocket::SyncSocket() : handle_(kInvalidHandle) {}
-
-SyncSocket::~SyncSocket() {
-  Close();
-}
-
 // static
 bool SyncSocket::CreatePair(SyncSocket* socket_a, SyncSocket* socket_b) {
   return CreatePairImpl(&socket_a->handle_, &socket_b->handle_, false);
 }
 
-// static
-SyncSocket::Handle SyncSocket::UnwrapHandle(
-    const TransitDescriptor& descriptor) {
-  return descriptor;
+void SyncSocket::Close() {
+  handle_.Close();
 }
 
-bool SyncSocket::PrepareTransitDescriptor(ProcessHandle peer_process_handle,
-                                          TransitDescriptor* descriptor) {
-  DCHECK(descriptor);
-  if (!::DuplicateHandle(GetCurrentProcess(), handle(), peer_process_handle,
-                         descriptor, 0, FALSE, DUPLICATE_SAME_ACCESS)) {
-    DPLOG(ERROR) << "Cannot duplicate socket handle for peer process.";
-    return false;
-  }
-  return true;
-}
-
-bool SyncSocket::Close() {
-  if (handle_ == kInvalidHandle)
-    return true;
-
-  const BOOL result = CloseHandle(handle_);
-  handle_ = kInvalidHandle;
-  return result == TRUE;
-}
-
-size_t SyncSocket::Send(const void* buffer, size_t length) {
+size_t SyncSocket::Send(span<const uint8_t> data) {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
-  DCHECK_GT(length, 0u);
-  DCHECK_LE(length, kMaxMessageLength);
-  DCHECK_NE(handle_, kInvalidHandle);
+  CHECK_LE(data.size(), kMaxMessageLength);
+  DCHECK(IsValid());
   size_t count = 0;
-  while (count < length) {
+  while (count < data.size()) {
     DWORD len;
-    DWORD chunk = GetNextChunkSize(count, length);
-    if (::WriteFile(handle_, static_cast<const char*>(buffer) + count, chunk,
-                    &len, NULL) == FALSE) {
+    const DWORD chunk_size = GetNextChunkSize(count, data.size());
+    auto data_chunk = data.subspan(count, chunk_size);
+    // SAFETY: The below static_cast is in range for DWORD because `data_chunk`
+    // is constructed with a DWORD length above from `chunk_size`.
+    if (::WriteFile(handle(), data_chunk.data(),
+                    static_cast<DWORD>(data_chunk.size()), &len,
+                    NULL) == FALSE) {
       return count;
     }
     count += len;
@@ -262,6 +242,15 @@ size_t SyncSocket::Send(const void* buffer, size_t length) {
   return count;
 }
 
+size_t SyncSocket::Send(const void* buffer, size_t length) {
+  return Send(make_span(static_cast<const uint8_t*>(buffer), length));
+}
+
+size_t SyncSocket::ReceiveWithTimeout(span<uint8_t> buffer, TimeDelta timeout) {
+  NOTIMPLEMENTED();
+  return 0;
+}
+
 size_t SyncSocket::ReceiveWithTimeout(void* buffer,
                                       size_t length,
                                       TimeDelta timeout) {
@@ -269,16 +258,19 @@ size_t SyncSocket::ReceiveWithTimeout(void* buffer,
   return 0;
 }
 
-size_t SyncSocket::Receive(void* buffer, size_t length) {
+size_t SyncSocket::Receive(span<uint8_t> buffer) {
   ScopedBlockingCall scoped_blocking_call(FROM_HERE, BlockingType::MAY_BLOCK);
-  DCHECK_GT(length, 0u);
-  DCHECK_LE(length, kMaxMessageLength);
-  DCHECK_NE(handle_, kInvalidHandle);
+  CHECK_LE(buffer.size(), kMaxMessageLength);
+  DCHECK(IsValid());
   size_t count = 0;
-  while (count < length) {
+  while (count < buffer.size()) {
     DWORD len;
-    DWORD chunk = GetNextChunkSize(count, length);
-    if (::ReadFile(handle_, static_cast<char*>(buffer) + count, chunk, &len,
+    const DWORD chunk_size = GetNextChunkSize(count, buffer.size());
+    auto data_chunk = buffer.subspan(count, chunk_size);
+    // SAFETY: The below static_cast is in range for DWORD because `data_chunk`
+    // is constructed with a DWORD length above from `chunk_size`.
+    if (::ReadFile(handle(), data_chunk.data(),
+                   static_cast<DWORD>(data_chunk.size()), &len,
                    NULL) == FALSE) {
       return count;
     }
@@ -287,30 +279,27 @@ size_t SyncSocket::Receive(void* buffer, size_t length) {
   return count;
 }
 
+size_t SyncSocket::Receive(void* buffer, size_t length) {
+  return Receive(make_span(static_cast<uint8_t*>(buffer), length));
+}
+
 size_t SyncSocket::Peek() {
   DWORD available = 0;
-  PeekNamedPipe(handle_, NULL, 0, NULL, &available, NULL);
+  PeekNamedPipe(handle(), NULL, 0, NULL, &available, NULL);
   return available;
 }
 
-SyncSocket::Handle SyncSocket::Release() {
-  Handle r = handle_;
-  handle_ = kInvalidHandle;
-  return r;
+bool SyncSocket::IsValid() const {
+  return handle_.is_valid();
 }
 
-CancelableSyncSocket::CancelableSyncSocket()
-    : shutdown_event_(base::WaitableEvent::ResetPolicy::MANUAL,
-                      base::WaitableEvent::InitialState::NOT_SIGNALED),
-      file_operation_(base::WaitableEvent::ResetPolicy::MANUAL,
-                      base::WaitableEvent::InitialState::NOT_SIGNALED) {}
+SyncSocket::Handle SyncSocket::handle() const {
+  return handle_.get();
+}
 
-CancelableSyncSocket::CancelableSyncSocket(Handle handle)
-    : SyncSocket(handle),
-      shutdown_event_(base::WaitableEvent::ResetPolicy::MANUAL,
-                      base::WaitableEvent::InitialState::NOT_SIGNALED),
-      file_operation_(base::WaitableEvent::ResetPolicy::MANUAL,
-                      base::WaitableEvent::InitialState::NOT_SIGNALED) {}
+SyncSocket::Handle SyncSocket::Release() {
+  return handle_.release();
+}
 
 bool CancelableSyncSocket::Shutdown() {
   // This doesn't shut down the pipe immediately, but subsequent Receive or Send
@@ -319,32 +308,43 @@ bool CancelableSyncSocket::Shutdown() {
   return true;
 }
 
-bool CancelableSyncSocket::Close() {
-  const bool result = SyncSocket::Close();
+void CancelableSyncSocket::Close() {
+  SyncSocket::Close();
   shutdown_event_.Reset();
-  return result;
+}
+
+size_t CancelableSyncSocket::Send(span<const uint8_t> data) {
+  static const DWORD kWaitTimeOutInMs = 500;
+  return CancelableFileOperation(&::WriteFile, handle(), data, &file_operation_,
+                                 &shutdown_event_, this, kWaitTimeOutInMs);
 }
 
 size_t CancelableSyncSocket::Send(const void* buffer, size_t length) {
-  static const DWORD kWaitTimeOutInMs = 500;
-  return CancelableFileOperation(
-      &::WriteFile, handle_, reinterpret_cast<const char*>(buffer), length,
-      &file_operation_, &shutdown_event_, this, kWaitTimeOutInMs);
+  return Send(make_span(static_cast<const uint8_t*>(buffer), length));
+}
+
+size_t CancelableSyncSocket::Receive(span<uint8_t> buffer) {
+  return CancelableFileOperation(&::ReadFile, handle(), buffer,
+                                 &file_operation_, &shutdown_event_, this,
+                                 INFINITE);
 }
 
 size_t CancelableSyncSocket::Receive(void* buffer, size_t length) {
-  return CancelableFileOperation(
-      &::ReadFile, handle_, reinterpret_cast<char*>(buffer), length,
-      &file_operation_, &shutdown_event_, this, INFINITE);
+  return Receive(make_span(static_cast<uint8_t*>(buffer), length));
+}
+
+size_t CancelableSyncSocket::ReceiveWithTimeout(span<uint8_t> buffer,
+                                                TimeDelta timeout) {
+  return CancelableFileOperation(&::ReadFile, handle(), buffer,
+                                 &file_operation_, &shutdown_event_, this,
+                                 static_cast<DWORD>(timeout.InMilliseconds()));
 }
 
 size_t CancelableSyncSocket::ReceiveWithTimeout(void* buffer,
                                                 size_t length,
                                                 TimeDelta timeout) {
-  return CancelableFileOperation(&::ReadFile, handle_,
-                                 reinterpret_cast<char*>(buffer), length,
-                                 &file_operation_, &shutdown_event_, this,
-                                 static_cast<DWORD>(timeout.InMilliseconds()));
+  return ReceiveWithTimeout(make_span(static_cast<uint8_t*>(buffer), length),
+                            std::move(timeout));
 }
 
 // static
