@@ -1,50 +1,58 @@
-// Copyright (c) 2012 The Chromium Authors. All rights reserved.
+// Copyright 2012 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "base/threading/thread.h"
 
+#include <memory>
 #include <type_traits>
+#include <utility>
 
-#include "base/bind.h"
-#include "base/bind_helpers.h"
-#include "base/lazy_instance.h"
+#include "base/dcheck_is_on.h"
+#include "base/functional/bind.h"
+#include "base/functional/callback_helpers.h"
 #include "base/location.h"
 #include "base/logging.h"
 #include "base/memory/ptr_util.h"
 #include "base/memory/scoped_refptr.h"
-#include "base/message_loop/message_loop_current.h"
 #include "base/message_loop/message_pump.h"
 #include "base/run_loop.h"
 #include "base/synchronization/waitable_event.h"
+#include "base/task/current_thread.h"
 #include "base/task/sequence_manager/sequence_manager_impl.h"
 #include "base/task/sequence_manager/task_queue.h"
-#include "base/task/simple_task_executor.h"
+#include "base/task/single_thread_task_runner.h"
 #include "base/third_party/dynamic_annotations/dynamic_annotations.h"
 #include "base/threading/thread_id_name_manager.h"
-#include "base/threading/thread_local.h"
 #include "base/threading/thread_restrictions.h"
-#include "base/threading/thread_task_runner_handle.h"
+#include "base/types/pass_key.h"
 #include "build/build_config.h"
+#include "third_party/abseil-cpp/absl/base/attributes.h"
 
-#if defined(OS_POSIX) && !defined(OS_NACL)
+#if (BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_NACL)) || BUILDFLAG(IS_FUCHSIA)
 #include "base/files/file_descriptor_watcher_posix.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 #endif
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
 #include "base/win/scoped_com_initializer.h"
 #endif
 
 namespace base {
 
+#if DCHECK_IS_ON()
 namespace {
 
 // We use this thread-local variable to record whether or not a thread exited
 // because its Stop method was called.  This allows us to catch cases where
 // MessageLoop::QuitWhenIdle() is called directly, which is unexpected when
 // using a Thread to setup and run a MessageLoop.
-base::LazyInstance<base::ThreadLocalBoolean>::Leaky lazy_tls_bool =
-    LAZY_INSTANCE_INITIALIZER;
+ABSL_CONST_INIT thread_local bool was_quit_properly = false;
+
+}  // namespace
+#endif
+
+namespace internal {
 
 class SequenceManagerThreadDelegate : public Thread::Delegate {
  public:
@@ -52,12 +60,14 @@ class SequenceManagerThreadDelegate : public Thread::Delegate {
       MessagePumpType message_pump_type,
       OnceCallback<std::unique_ptr<MessagePump>()> message_pump_factory)
       : sequence_manager_(
-            sequence_manager::internal::SequenceManagerImpl::CreateUnbound(
+            sequence_manager::internal::CreateUnboundSequenceManagerImpl(
+                PassKey<base::internal::SequenceManagerThreadDelegate>(),
                 sequence_manager::SequenceManager::Settings::Builder()
                     .SetMessagePumpType(message_pump_type)
                     .Build())),
         default_task_queue_(sequence_manager_->CreateTaskQueue(
-            sequence_manager::TaskQueue::Spec("default_tq"))),
+            sequence_manager::TaskQueue::Spec(
+                sequence_manager::QueueName::DEFAULT_TQ))),
         message_pump_factory_(std::move(message_pump_factory)) {
     sequence_manager_->SetDefaultTaskRunner(default_task_queue_->task_runner());
   }
@@ -83,30 +93,50 @@ class SequenceManagerThreadDelegate : public Thread::Delegate {
     return sequence_manager_->GetTaskRunner();
   }
 
-  void BindToCurrentThread(TimerSlack timer_slack) override {
+  void BindToCurrentThread() override {
     sequence_manager_->BindToMessagePump(
         std::move(message_pump_factory_).Run());
-    sequence_manager_->SetTimerSlack(timer_slack);
-    simple_task_executor_.emplace(sequence_manager_.get(),
-                                  GetDefaultTaskRunner());
   }
 
  private:
   std::unique_ptr<sequence_manager::internal::SequenceManagerImpl>
       sequence_manager_;
-  scoped_refptr<sequence_manager::TaskQueue> default_task_queue_;
+  sequence_manager::TaskQueue::Handle default_task_queue_;
   OnceCallback<std::unique_ptr<MessagePump>()> message_pump_factory_;
-  base::Optional<SimpleTaskExecutor> simple_task_executor_;
 };
 
-}  // namespace
+}  // namespace internal
 
 Thread::Options::Options() = default;
 
 Thread::Options::Options(MessagePumpType type, size_t size)
     : message_pump_type(type), stack_size(size) {}
 
-Thread::Options::Options(Options&& other) = default;
+Thread::Options::Options(ThreadType thread_type) : thread_type(thread_type) {}
+
+Thread::Options::Options(Options&& other)
+    : message_pump_type(std::move(other.message_pump_type)),
+      delegate(std::move(other.delegate)),
+      message_pump_factory(std::move(other.message_pump_factory)),
+      stack_size(std::move(other.stack_size)),
+      thread_type(std::move(other.thread_type)),
+      joinable(std::move(other.joinable)) {
+  other.moved_from = true;
+}
+
+Thread::Options& Thread::Options::operator=(Thread::Options&& other) {
+  DCHECK_NE(this, &other);
+
+  message_pump_type = std::move(other.message_pump_type);
+  delegate = std::move(other.delegate);
+  message_pump_factory = std::move(other.message_pump_factory);
+  stack_size = std::move(other.stack_size);
+  thread_type = std::move(other.thread_type);
+  joinable = std::move(other.joinable);
+  other.moved_from = true;
+
+  return *this;
+}
 
 Thread::Options::~Options() = default;
 
@@ -131,20 +161,21 @@ bool Thread::Start() {
   DCHECK(owning_sequence_checker_.CalledOnValidSequence());
 
   Options options;
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   if (com_status_ == STA)
     options.message_pump_type = MessagePumpType::UI;
 #endif
-  return StartWithOptions(options);
+  return StartWithOptions(std::move(options));
 }
 
-bool Thread::StartWithOptions(const Options& options) {
+bool Thread::StartWithOptions(Options options) {
+  DCHECK(options.IsValid());
   DCHECK(owning_sequence_checker_.CalledOnValidSequence());
   DCHECK(!delegate_);
   DCHECK(!IsRunning());
   DCHECK(!stopping_) << "Starting a non-joinable thread a second time? That's "
                      << "not allowed!";
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   DCHECK((com_status_ != STA) ||
          (options.message_pump_type == MessagePumpType::UI));
 #endif
@@ -155,16 +186,14 @@ bool Thread::StartWithOptions(const Options& options) {
 
   SetThreadWasQuitProperly(false);
 
-  timer_slack_ = options.timer_slack;
-
   if (options.delegate) {
     DCHECK(!options.message_pump_factory);
-    delegate_ = WrapUnique(options.delegate);
+    delegate_ = std::move(options.delegate);
   } else if (options.message_pump_factory) {
-    delegate_ = std::make_unique<SequenceManagerThreadDelegate>(
+    delegate_ = std::make_unique<internal::SequenceManagerThreadDelegate>(
         MessagePumpType::CUSTOM, options.message_pump_factory);
   } else {
-    delegate_ = std::make_unique<SequenceManagerThreadDelegate>(
+    delegate_ = std::make_unique<internal::SequenceManagerThreadDelegate>(
         options.message_pump_type,
         BindOnce([](MessagePumpType type) { return MessagePump::Create(type); },
                  options.message_pump_type));
@@ -177,12 +206,13 @@ bool Thread::StartWithOptions(const Options& options) {
   // fixed).
   {
     AutoLock lock(thread_lock_);
-    bool success =
-        options.joinable
-            ? PlatformThread::CreateWithPriority(options.stack_size, this,
-                                                 &thread_, options.priority)
-            : PlatformThread::CreateNonJoinableWithPriority(
-                  options.stack_size, this, options.priority);
+    bool success = options.joinable
+                       ? PlatformThread::CreateWithType(
+                             options.stack_size, this, &thread_,
+                             options.thread_type, options.message_pump_type)
+                       : PlatformThread::CreateNonJoinableWithType(
+                             options.stack_size, this, options.thread_type,
+                             options.message_pump_type);
     if (!success) {
       DLOG(ERROR) << "failed to create thread";
       return false;
@@ -274,9 +304,11 @@ void Thread::DetachFromSequence() {
 }
 
 PlatformThreadId Thread::GetThreadId() const {
-  // If the thread is created but not started yet, wait for |id_| being ready.
-  base::ScopedAllowBaseSyncPrimitivesOutsideBlockingScope allow_wait;
-  id_event_.Wait();
+  if (!id_event_.IsSignaled()) {
+    // If the thread is created but not started yet, wait for |id_| being ready.
+    base::ScopedAllowBaseSyncPrimitivesOutsideBlockingScope allow_wait;
+    id_event_.Wait();
+  }
   return id_;
 }
 
@@ -307,16 +339,18 @@ void Thread::Run(RunLoop* run_loop) {
 
 // static
 void Thread::SetThreadWasQuitProperly(bool flag) {
-  lazy_tls_bool.Pointer()->Set(flag);
+#if DCHECK_IS_ON()
+  was_quit_properly = flag;
+#endif
 }
 
 // static
 bool Thread::GetThreadWasQuitProperly() {
-  bool quit_properly = true;
 #if DCHECK_IS_ON()
-  quit_properly = lazy_tls_bool.Pointer()->Get();
+  return was_quit_properly;
+#else
+  return true;
 #endif
-  return quit_properly;
 }
 
 void Thread::ThreadMain() {
@@ -337,21 +371,20 @@ void Thread::ThreadMain() {
 
   // Lazily initialize the |message_loop| so that it can run on this thread.
   DCHECK(delegate_);
-  // This binds MessageLoopCurrent and ThreadTaskRunnerHandle.
-  delegate_->BindToCurrentThread(timer_slack_);
-  DCHECK(MessageLoopCurrent::Get());
-  DCHECK(ThreadTaskRunnerHandle::IsSet());
-
-#if defined(OS_POSIX) && !defined(OS_NACL)
+  // This binds CurrentThread and SingleThreadTaskRunner::CurrentDefaultHandle.
+  delegate_->BindToCurrentThread();
+  DCHECK(CurrentThread::Get());
+  DCHECK(SingleThreadTaskRunner::HasCurrentDefault());
+#if (BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_NACL)) || BUILDFLAG(IS_FUCHSIA)
   // Allow threads running a MessageLoopForIO to use FileDescriptorWatcher API.
   std::unique_ptr<FileDescriptorWatcher> file_descriptor_watcher;
-  if (MessageLoopCurrentForIO::IsSet()) {
-    file_descriptor_watcher.reset(
-        new FileDescriptorWatcher(delegate_->GetDefaultTaskRunner()));
+  if (CurrentIOThread::IsSet()) {
+    file_descriptor_watcher = std::make_unique<FileDescriptorWatcher>(
+        delegate_->GetDefaultTaskRunner());
   }
-#endif
+#endif  // (BUILDFLAG(IS_POSIX) && !BUILDFLAG(IS_NACL)) || BUILDFLAG(IS_FUCHSIA)
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   std::unique_ptr<win::ScopedCOMInitializer> com_initializer;
   if (com_status_ != NONE) {
     com_initializer.reset(
@@ -383,7 +416,7 @@ void Thread::ThreadMain() {
   // Let the thread do extra cleanup.
   CleanUp();
 
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
   com_initializer.reset();
 #endif
 
