@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright 2010 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,14 +6,13 @@
 
 #include <objbase.h>
 
+#include <string_view>
+
+#include "base/containers/contains.h"
 #include "base/files/file_path.h"
-#include "base/macros.h"
 #include "base/scoped_native_library.h"
-#include "base/stl_util.h"
-#include "base/strings/string16.h"
 #include "base/strings/string_util.h"
 #include "base/win/scoped_co_mem.h"
-#include "base/win/win_client_metrics.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace base {
@@ -26,12 +25,14 @@ namespace {
 class ThreadLocaleSaver {
  public:
   ThreadLocaleSaver() : original_locale_id_(GetThreadLocale()) {}
+
+  ThreadLocaleSaver(const ThreadLocaleSaver&) = delete;
+  ThreadLocaleSaver& operator=(const ThreadLocaleSaver&) = delete;
+
   ~ThreadLocaleSaver() { SetThreadLocale(original_locale_id_); }
 
  private:
   LCID original_locale_id_;
-
-  DISALLOW_COPY_AND_ASSIGN(ThreadLocaleSaver);
 };
 
 }  // namespace
@@ -43,19 +44,11 @@ TEST(BaseWinUtilTest, TestIsUACEnabled) {
 }
 
 TEST(BaseWinUtilTest, TestGetUserSidString) {
-  string16 user_sid;
+  std::wstring user_sid;
   EXPECT_TRUE(GetUserSidString(&user_sid));
   EXPECT_TRUE(!user_sid.empty());
 }
 
-TEST(BaseWinUtilTest, TestGetNonClientMetrics) {
-  NONCLIENTMETRICS_XP metrics = {0};
-  GetNonClientMetrics(&metrics);
-  EXPECT_GT(metrics.cbSize, 0u);
-  EXPECT_GT(metrics.iScrollWidth, 0);
-  EXPECT_GT(metrics.iScrollHeight, 0);
-}
-
 TEST(BaseWinUtilTest, TestGetLoadedModulesSnapshot) {
   std::vector<HMODULE> snapshot;
 
@@ -66,11 +59,11 @@ TEST(BaseWinUtilTest, TestGetLoadedModulesSnapshot) {
 
   // Load in a new module. Pick zipfldr.dll as it is present from WinXP to
   // Win10, including ARM64 Win10, and yet rarely used.
-  const char16 dll_name[] = FILE_PATH_LITERAL("zipfldr.dll");
-  ASSERT_EQ(NULL, ::GetModuleHandle(as_wcstr(dll_name)));
+  const FilePath::CharType dll_name[] = FILE_PATH_LITERAL("zipfldr.dll");
+  ASSERT_EQ(nullptr, ::GetModuleHandle(dll_name));
 
   ScopedNativeLibrary new_dll((FilePath(dll_name)));
-  ASSERT_NE(static_cast<HMODULE>(NULL), new_dll.get());
+  ASSERT_NE(static_cast<HMODULE>(nullptr), new_dll.get());
   ASSERT_TRUE(GetLoadedModulesSnapshot(::GetCurrentProcess(), &snapshot));
   ASSERT_GT(snapshot.size(), original_snapshot_size);
   ASSERT_TRUE(Contains(snapshot, new_dll.get()));
@@ -83,25 +76,24 @@ TEST(BaseWinUtilTest, TestUint32ToInvalidHandle) {
   EXPECT_EQ(INVALID_HANDLE_VALUE, Uint32ToHandle(invalid_handle));
 }
 
-TEST(BaseWinUtilTest, String16FromGUID) {
+TEST(BaseWinUtilTest, WStringFromGUID) {
   const GUID kGuid = {0x7698f759,
                       0xf5b0,
                       0x4328,
                       {0x92, 0x38, 0xbd, 0x70, 0x8a, 0x6d, 0xc9, 0x63}};
-  const base::StringPiece16 kGuidStr(
-      STRING16_LITERAL("{7698F759-F5B0-4328-9238-BD708A6DC963}"));
-  auto guid_string16 = String16FromGUID(kGuid);
-  EXPECT_EQ(guid_string16, kGuidStr);
+  const std::wstring_view kGuidStr = L"{7698F759-F5B0-4328-9238-BD708A6DC963}";
+  auto guid_wstring = WStringFromGUID(kGuid);
+  EXPECT_EQ(guid_wstring, kGuidStr);
   wchar_t guid_wchar[39];
-  ::StringFromGUID2(kGuid, guid_wchar, base::size(guid_wchar));
-  EXPECT_STREQ(as_wcstr(guid_string16), guid_wchar);
+  ::StringFromGUID2(kGuid, guid_wchar, std::size(guid_wchar));
+  EXPECT_STREQ(guid_wstring.c_str(), guid_wchar);
   ScopedCoMem<OLECHAR> clsid_string;
   ::StringFromCLSID(kGuid, &clsid_string);
-  EXPECT_STREQ(as_wcstr(guid_string16), clsid_string.get());
+  EXPECT_STREQ(guid_wstring.c_str(), clsid_string.get());
 }
 
 TEST(BaseWinUtilTest, GetWindowObjectName) {
-  base::string16 created_desktop_name(STRING16_LITERAL("test_desktop"));
+  std::wstring created_desktop_name(L"test_desktop");
   HDESK desktop_handle =
       ::CreateDesktop(created_desktop_name.c_str(), nullptr, nullptr, 0,
                       DESKTOP_CREATEWINDOW | DESKTOP_READOBJECTS |
@@ -117,13 +109,15 @@ TEST(BaseWinUtilTest, IsRunningUnderDesktopName) {
   HDESK thread_desktop = ::GetThreadDesktop(::GetCurrentThreadId());
 
   ASSERT_NE(thread_desktop, nullptr);
-  base::string16 desktop_name = GetWindowObjectName(thread_desktop);
+  std::wstring desktop_name = GetWindowObjectName(thread_desktop);
 
   EXPECT_TRUE(IsRunningUnderDesktopName(desktop_name));
-  EXPECT_TRUE(IsRunningUnderDesktopName(base::ToLowerASCII(desktop_name)));
-  EXPECT_TRUE(IsRunningUnderDesktopName(base::ToUpperASCII(desktop_name)));
-  EXPECT_FALSE(IsRunningUnderDesktopName(
-      desktop_name + STRING16_LITERAL("_non_existent_desktop_name")));
+  EXPECT_TRUE(IsRunningUnderDesktopName(
+      AsWString(ToLowerASCII(AsStringPiece16(desktop_name)))));
+  EXPECT_TRUE(IsRunningUnderDesktopName(
+      AsWString(ToUpperASCII(AsStringPiece16(desktop_name)))));
+  EXPECT_FALSE(
+      IsRunningUnderDesktopName(desktop_name + L"_non_existent_desktop_name"));
 }
 
 }  // namespace win
