@@ -1,4 +1,4 @@
-// Copyright 2011 The Chromium Authors. All rights reserved.
+// Copyright 2011 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,19 +14,19 @@
 #include <string>
 #include <vector>
 
-#include "base/logging.h"
+#include "base/check.h"
+#include "base/containers/flat_set.h"
 #include "base/memory/ptr_util.h"
-#include "base/values.h"
+#include "base/memory/raw_ptr.h"
 #include "cc/base/region.h"
 #include "cc/base/synced_property.h"
 #include "cc/cc_export.h"
 #include "cc/debug/layer_tree_debug_state.h"
+#include "cc/input/hit_test_opaqueness.h"
 #include "cc/input/input_handler.h"
 #include "cc/layers/draw_mode.h"
 #include "cc/layers/draw_properties.h"
 #include "cc/layers/layer_collections.h"
-#include "cc/layers/layer_impl_test_properties.h"
-#include "cc/layers/layer_position_constraint.h"
 #include "cc/layers/performance_properties.h"
 #include "cc/layers/render_surface_impl.h"
 #include "cc/layers/touch_action_region.h"
@@ -34,32 +34,28 @@
 #include "cc/tiles/tile_priority.h"
 #include "cc/trees/target_property.h"
 #include "components/viz/common/quads/shared_quad_state.h"
+#include "components/viz/common/surfaces/region_capture_bounds.h"
 #include "third_party/skia/include/core/SkColor.h"
+#include "ui/gfx/display_color_spaces.h"
 #include "ui/gfx/geometry/point3_f.h"
+#include "ui/gfx/geometry/point_f.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/rect_f.h"
-#include "ui/gfx/geometry/scroll_offset.h"
-#include "ui/gfx/transform.h"
-
-namespace base {
-namespace trace_event {
-class TracedValue;
-}
-class DictionaryValue;
-}
+#include "ui/gfx/geometry/transform.h"
+#include "ui/gfx/geometry/vector2d_f.h"
 
 namespace viz {
 class ClientResourceProvider;
-class RenderPass;
+class CompositorRenderPass;
 }
 
 namespace cc {
 
 class AppendQuadsData;
+struct LayerDebugInfo;
 class LayerTreeImpl;
 class MicroBenchmarkImpl;
 class PrioritizedTile;
-class ScrollbarLayerImplBase;
 class SimpleEnclosedRegion;
 class Tile;
 
@@ -90,7 +86,7 @@ class CC_EXPORT LayerImpl {
   bool IsActive() const;
 
   void SetHasTransformNode(bool val) { has_transform_node_ = val; }
-  bool has_transform_node() { return has_transform_node_; }
+  bool has_transform_node() const { return has_transform_node_; }
 
   void set_property_tree_sequence_number(int sequence_number) {}
 
@@ -114,14 +110,6 @@ class CC_EXPORT LayerImpl {
     return offset_to_transform_parent_;
   }
 
-  void SetShouldFlattenScreenSpaceTransformFromPropertyTree(
-      bool should_flatten) {
-    should_flatten_screen_space_transform_from_property_tree_ = should_flatten;
-  }
-  bool should_flatten_screen_space_transform_from_property_tree() const {
-    return should_flatten_screen_space_transform_from_property_tree_;
-  }
-
   bool is_clipped() const { return draw_properties_.is_clipped; }
 
   LayerTreeImpl* layer_tree_impl() const { return layer_tree_impl_; }
@@ -148,7 +136,7 @@ class CC_EXPORT LayerImpl {
   // the layer is destroyed.
   virtual bool WillDraw(DrawMode draw_mode,
                         viz::ClientResourceProvider* resource_provider);
-  virtual void AppendQuads(viz::RenderPass* render_pass,
+  virtual void AppendQuads(viz::CompositorRenderPass* render_pass,
                            AppendQuadsData* append_quads_data) {}
   virtual void DidDraw(viz::ClientResourceProvider* resource_provider);
 
@@ -165,34 +153,33 @@ class CC_EXPORT LayerImpl {
 
   virtual void NotifyTileStateChanged(const Tile* tile) {}
 
-  virtual ScrollbarLayerImplBase* ToScrollbarLayer();
+  virtual bool IsScrollbarLayer() const;
+
+  bool IsScrollerOrScrollbar() const;
 
   // Returns true if this layer has content to draw.
   void SetDrawsContent(bool draws_content);
-  bool DrawsContent() const { return draws_content_; }
+  bool draws_content() const { return draws_content_; }
 
-  // Make the layer hit testable.
-  void SetHitTestable(bool should_hit_test);
+  void SetHitTestOpaqueness(HitTestOpaqueness opaqueness);
   bool HitTestable() const;
+  bool OpaqueToHitTest() const;
 
-  LayerImplTestProperties* test_properties() {
-    if (!test_properties_)
-      test_properties_.reset(new LayerImplTestProperties(this));
-    return test_properties_.get();
+  void SetBackgroundColor(SkColor4f background_color);
+  SkColor4f background_color() const { return background_color_; }
+  void SetSafeOpaqueBackgroundColor(SkColor4f background_color);
+  SkColor4f safe_opaque_background_color() const {
+    // Layer::SafeOpaqueBackgroundColor() should ensure this.
+    DCHECK_EQ(contents_opaque(), safe_opaque_background_color_.isOpaque());
+    return safe_opaque_background_color_;
   }
 
-  void SetBackgroundColor(SkColor background_color);
-  SkColor background_color() const { return background_color_; }
-  void SetSafeOpaqueBackgroundColor(SkColor background_color);
-  // If contents_opaque(), return an opaque color else return a
-  // non-opaque color.  Tries to return background_color(), if possible.
-  SkColor SafeOpaqueBackgroundColor() const;
-
-  void SetMasksToBounds(bool masks_to_bounds);
-  bool masks_to_bounds() const { return masks_to_bounds_; }
-
+  // See Layer::SetContentsOpaque() and SetContentsOpaqueForText() for the
+  // relationship between the two flags.
   void SetContentsOpaque(bool opaque);
   bool contents_opaque() const { return contents_opaque_; }
+  void SetContentsOpaqueForText(bool opaque);
+  bool contents_opaque_for_text() const { return contents_opaque_for_text_; }
 
   float Opacity() const;
 
@@ -200,23 +187,10 @@ class CC_EXPORT LayerImpl {
   void SetElementId(ElementId element_id);
   ElementId element_id() const { return element_id_; }
 
-  void SetMirrorCount(int mirror_count);
-  int mirror_count() const { return mirror_count_; }
-
   bool IsAffectedByPageScale() const;
 
   bool Is3dSorted() const { return GetSortingContextId() != 0; }
 
-  void SetUseParentBackfaceVisibility(bool use) {
-    use_parent_backface_visibility_ = use;
-  }
-  bool use_parent_backface_visibility() const {
-    return use_parent_backface_visibility_;
-  }
-
-  bool IsResizedByBrowserControls() const;
-  void SetIsResizedByBrowserControls(bool resized);
-
   void SetShouldCheckBackfaceVisibility(bool should_check_backface_visibility) {
     should_check_backface_visibility_ = should_check_backface_visibility;
   }
@@ -240,8 +214,6 @@ class CC_EXPORT LayerImpl {
     return performance_properties_;
   }
 
-  bool CanUseLCDText() const;
-
   // Setter for draw_properties_.
   void set_visible_layer_rect(const gfx::Rect& visible_rect) {
     draw_properties_.visible_layer_rect = visible_rect;
@@ -257,8 +229,8 @@ class CC_EXPORT LayerImpl {
     return draw_properties_.screen_space_transform_is_animating;
   }
   gfx::Rect clip_rect() const { return draw_properties_.clip_rect; }
-  gfx::Rect drawable_content_rect() const {
-    return draw_properties_.drawable_content_rect;
+  gfx::Rect visible_drawable_content_rect() const {
+    return draw_properties_.visible_drawable_content_rect;
   }
   gfx::Rect visible_layer_rect() const {
     return draw_properties_.visible_layer_rect;
@@ -270,54 +242,48 @@ class CC_EXPORT LayerImpl {
 
   void SetBounds(const gfx::Size& bounds);
   gfx::Size bounds() const;
-  // Like bounds() but doesn't snap to int. Lossy on giant pages (e.g. millions
-  // of pixels) due to use of single precision float.
-  gfx::SizeF BoundsForScrolling() const;
 
-  // Viewport bounds delta are only used for viewport layers and account for
-  // changes in the viewport layers from browser controls and page scale
-  // factors. These deltas are only set on the active tree.
-  // TODO(bokan): These methods should be unneeded now that LTHI sets these
-  // directly on the property trees.
-  void SetViewportBoundsDelta(const gfx::Vector2dF& bounds_delta);
-  gfx::Vector2dF ViewportBoundsDelta() const;
-
-  void SetViewportLayerType(ViewportLayerType type) {
-    // Once set as a viewport layer type, the viewport type should not change.
-    DCHECK(viewport_layer_type() == NOT_VIEWPORT_LAYER ||
-           viewport_layer_type() == type);
-    viewport_layer_type_ = type;
-  }
-  ViewportLayerType viewport_layer_type() const {
-    return static_cast<ViewportLayerType>(viewport_layer_type_);
-  }
-  bool is_viewport_layer_type() const {
-    return viewport_layer_type() != NOT_VIEWPORT_LAYER;
+  void set_is_inner_viewport_scroll_layer() {
+    is_inner_viewport_scroll_layer_ = true;
   }
 
-  void SetCurrentScrollOffset(const gfx::ScrollOffset& scroll_offset);
-  gfx::ScrollOffset CurrentScrollOffset() const;
-
-  gfx::ScrollOffset MaxScrollOffset() const;
-  gfx::ScrollOffset ClampScrollOffsetToLimits(gfx::ScrollOffset offset) const;
-  gfx::Vector2dF ClampScrollToMaxScrollOffset();
+  void SetCurrentScrollOffset(const gfx::PointF& scroll_offset);
 
   // Returns the delta of the scroll that was outside of the bounds of the
   // initial scroll
   gfx::Vector2dF ScrollBy(const gfx::Vector2dF& scroll);
 
-  // Marks this layer as being scrollable and needing an associated scroll node.
-  // The scroll node's bounds and container_bounds will be kept in sync with
-  // this layer.
-  void SetScrollable(const gfx::Size& bounds);
-  gfx::Size scroll_container_bounds() const { return scroll_container_bounds_; }
-  bool scrollable() const { return scrollable_; }
+  // Called during a commit or activation, after the property trees are pushed.
+  // It detects changes of scrollable status and scroll container size in the
+  // scroll property, and invalidate scrollbar geometries etc. for the changes.
+  void UpdateScrollable();
+
+  // Some properties on the LayerImpl are rarely set, and so are bundled
+  // under a single unique_ptr.
+  struct RareProperties {
+    // The bounds of elements marked for potential region capture, stored in
+    // the coordinate space of this layer.
+    viz::RegionCaptureBounds capture_bounds;
+    Region non_fast_scrollable_region;
+    Region wheel_event_handler_region;
+  };
+
+  RareProperties& EnsureRareProperties() {
+    if (!rare_properties_)
+      rare_properties_ = std::make_unique<RareProperties>();
+
+    return *rare_properties_;
+  }
+
+  void ResetRareProperties() { rare_properties_.reset(); }
 
   void SetNonFastScrollableRegion(const Region& region) {
-    non_fast_scrollable_region_ = region;
+    if (rare_properties_ || !region.IsEmpty())
+      EnsureRareProperties().non_fast_scrollable_region = region;
   }
   const Region& non_fast_scrollable_region() const {
-    return non_fast_scrollable_region_;
+    return rare_properties_ ? rare_properties_->non_fast_scrollable_region
+                            : Region::Empty();
   }
 
   void SetTouchActionRegion(TouchActionRegion);
@@ -329,18 +295,30 @@ class CC_EXPORT LayerImpl {
     return !touch_action_region_.IsEmpty();
   }
 
+  void SetCaptureBounds(viz::RegionCaptureBounds bounds);
+  const viz::RegionCaptureBounds* capture_bounds() const {
+    return rare_properties_ ? &rare_properties_->capture_bounds : nullptr;
+  }
+
   // Set or get the region that contains wheel event handler.
   // The |wheel_event_handler_region| specify the area where wheel event handler
   // could block impl scrolling.
   void SetWheelEventHandlerRegion(const Region& wheel_event_handler_region) {
-    wheel_event_handler_region_ = wheel_event_handler_region;
+    if (rare_properties_ || !wheel_event_handler_region.IsEmpty()) {
+      EnsureRareProperties().wheel_event_handler_region =
+          wheel_event_handler_region;
+    }
   }
   const Region& wheel_event_handler_region() const {
-    return wheel_event_handler_region_;
+    return rare_properties_ ? rare_properties_->wheel_event_handler_region
+                            : Region::Empty();
   }
 
+  // The main thread may commit multiple times before the impl thread actually
+  // draws, so we need to accumulate (i.e. union) any update changes that have
+  // occurred on the main thread until we draw.
   // Note this rect is in layer space (not content space).
-  void SetUpdateRect(const gfx::Rect& update_rect);
+  void UnionUpdateRect(const gfx::Rect& update_rect);
   const gfx::Rect& update_rect() const { return update_rect_; }
 
   // Denotes an area that is damaged and needs redraw. This is in the layer's
@@ -348,11 +326,6 @@ class CC_EXPORT LayerImpl {
   // as appropriate.
   virtual gfx::Rect GetDamageRect() const;
 
-  virtual std::unique_ptr<base::DictionaryValue> LayerAsJson() const;
-  // TODO(pdr): This should be removed because there is no longer a tree
-  // of layers, only a list.
-  std::unique_ptr<base::DictionaryValue> LayerTreeAsJson();
-
   // This includes |layer_property_changed_not_from_property_trees_| and
   // property_trees changes.
   bool LayerPropertyChanged() const;
@@ -387,7 +360,8 @@ class CC_EXPORT LayerImpl {
   // ReleaseTileResources call.
   virtual void RecreateTileResources();
 
-  virtual std::unique_ptr<LayerImpl> CreateLayerImpl(LayerTreeImpl* tree_impl);
+  virtual std::unique_ptr<LayerImpl> CreateLayerImpl(
+      LayerTreeImpl* tree_impl) const;
   virtual void PushPropertiesTo(LayerImpl* layer);
 
   // Internal to property tree construction (which only happens in tests on a
@@ -405,14 +379,15 @@ class CC_EXPORT LayerImpl {
   // Mark a layer on pending tree that needs to push its properties to the
   // active tree. These properties should not be changed during pending tree
   // lifetime, and only changed by being pushed from the main thread. There are
-  // two cases where this function needs to be called: when main thread layer
-  // has properties that need to be pushed, or when a new LayerImpl is created
-  // on pending tree when syncing layers from main thread.
+  // three cases where this function needs to be called: when main thread layer
+  // has properties that need to be pushed, when a new LayerImpl is created
+  // on pending tree when syncing layers from main thread, or when we recompute
+  // visible layer properties on the pending tree.
   void SetNeedsPushProperties();
 
   virtual void RunMicroBenchmark(MicroBenchmarkImpl* benchmark);
 
-  void SetDebugInfo(std::unique_ptr<base::trace_event::TracedValue> debug_info);
+  void UpdateDebugInfo(LayerDebugInfo* debug_info);
 
   void set_contributes_to_drawn_render_surface(bool is_member) {
     contributes_to_drawn_render_surface_ = is_member;
@@ -422,10 +397,6 @@ class CC_EXPORT LayerImpl {
     return contributes_to_drawn_render_surface_;
   }
 
-  bool is_scrollbar() const { return is_scrollbar_; }
-
-  void set_is_scrollbar(bool is_scrollbar) { is_scrollbar_ = is_scrollbar; }
-
   void set_may_contain_video(bool yes) { may_contain_video_ = yes; }
   bool may_contain_video() const { return may_contain_video_; }
 
@@ -438,28 +409,36 @@ class CC_EXPORT LayerImpl {
   // for layers that provide it.
   virtual Region GetInvalidationRegionForDebugging();
 
+  // Returns the visible rect that is used by damage tracker. This damage rect
+  // is computed as an eclosing rect of actual content size x transform to
+  // target space, which can be different from visible_drawing_content_rect. For
+  // example, the actual tile size of picture layer in the target surface can be
+  // bigger when e.g. the layer's raster scale is different from the scale of
+  // the layer's draw transform.
   // If you override this, and are making use of
   // PopulateScaledSharedQuadState(), make sure you call
-  // GetScaledEnclosingRectInTargetSpace(). See comment for
+  // GetScaledEnclosingVisibleRectInTargetSpace(). See comment for
   // PopulateScaledSharedQuadState().
-  virtual gfx::Rect GetEnclosingRectInTargetSpace() const;
+  virtual gfx::Rect GetEnclosingVisibleRectInTargetSpace() const;
 
-  // Returns the bounds of this layer in target space when scaled by |scale|.
-  // This function scales in the same way as
+  // Returns the visible bounds of this layer in target space when scaled by
+  // |scale|.  This function scales in the same way as
   // PopulateScaledSharedQuadStateQuadState(). See
   // PopulateScaledSharedQuadStateQuadState() for more details.
-  gfx::Rect GetScaledEnclosingRectInTargetSpace(float scale) const;
+  gfx::Rect GetScaledEnclosingVisibleRectInTargetSpace(float scale) const;
 
-  float GetIdealContentsScale() const;
+  // GetIdealContentsScale() returns the ideal 2D scale, clamped to not exceed
+  // GetPreferredRasterScale().
+  // GetIdealContentsScaleKey() returns the maximum component, a fallback to
+  // uniform scale for callers that don't support 2d scales yet.
+  // TODO(crbug.com/1196414): Remove GetIdealContentsScaleKey() in favor of
+  // GetIdealContentsScale().
+  gfx::Vector2dF GetIdealContentsScale() const;
+  float GetIdealContentsScaleKey() const;
 
   void NoteLayerPropertyChanged();
   void NoteLayerPropertyChangedFromPropertyTrees();
 
-  void SetHasWillChangeTransformHint(bool has_will_change);
-  bool has_will_change_transform_hint() const {
-    return has_will_change_transform_hint_;
-  }
-
   ElementListType GetElementTypeForAnimation() const;
 
   void set_needs_show_scrollbars(bool yes) { needs_show_scrollbars_ = yes; }
@@ -475,6 +454,18 @@ class CC_EXPORT LayerImpl {
   // TODO(sunxd): Remove this function and replace it with visitor pattern.
   virtual bool is_surface_layer() const;
 
+  int CalculateJitter();
+
+  std::string DebugName() const;
+
+  virtual gfx::ContentColorUsage GetContentColorUsage() const;
+
+  virtual void NotifyKnownResourceIdsBeforeAppendQuads(
+      const base::flat_set<viz::ViewTransitionElementResourceId>&
+          known_resource_ids) {}
+
+  virtual viz::ViewTransitionElementResourceId ViewTransitionResourceId() const;
+
  protected:
   // When |will_always_push_properties| is true, the layer will not itself set
   // its SetNeedsPushProperties() state, as it expects to be always pushed to
@@ -484,44 +475,48 @@ class CC_EXPORT LayerImpl {
             bool will_always_push_properties = false);
 
   // Get the color and size of the layer's debug border.
-  virtual void GetDebugBorderProperties(SkColor* color, float* width) const;
+  virtual void GetDebugBorderProperties(SkColor4f* color, float* width) const;
 
-  void AppendDebugBorderQuad(viz::RenderPass* render_pass,
+  void AppendDebugBorderQuad(viz::CompositorRenderPass* render_pass,
                              const gfx::Rect& quad_rect,
                              const viz::SharedQuadState* shared_quad_state,
                              AppendQuadsData* append_quads_data) const;
-  void AppendDebugBorderQuad(viz::RenderPass* render_pass,
+  void AppendDebugBorderQuad(viz::CompositorRenderPass* render_pass,
                              const gfx::Rect& quad_rect,
                              const viz::SharedQuadState* shared_quad_state,
                              AppendQuadsData* append_quads_data,
-                             SkColor color,
+                             SkColor4f color,
                              float width) const;
 
+  static float GetPreferredRasterScale(
+      gfx::Vector2dF raster_space_scale_factor);
+
  private:
   void ValidateQuadResourcesInternal(viz::DrawQuad* quad) const;
+  gfx::Transform GetScaledDrawTransform(float layer_to_content_scale) const;
 
   virtual const char* LayerTypeAsString() const;
 
   const int layer_id_;
-  LayerTreeImpl* const layer_tree_impl_;
+  const raw_ptr<LayerTreeImpl> layer_tree_impl_;
   const bool will_always_push_properties_ : 1;
 
-  std::unique_ptr<LayerImplTestProperties> test_properties_;
-
   // Properties synchronized from the associated Layer.
   gfx::Size bounds_;
 
   gfx::Vector2dF offset_to_transform_parent_;
 
-  // Size of the scroll container that this layer scrolls in.
+  // These fields are copies of |container_bounds|, |bounds| and |scrollable|
+  // fields in the associated ScrollNode, and are updated in UpdateScrollable().
+  // The copy is for change detection only.
+  // TODO(wangxianzhu): Actually we only need scroll_container_bounds_ in
+  // pre-CompositeAfterPaint where the scroll node is associated with the
+  // scrolling contents layer, and only need scroll_contents_bounds_ in
+  // CompositeAfterPaint where the scroll node is associated with the scroll
+  // container layer. Remove scroll_container_bounds_ when we launch CAP.
   gfx::Size scroll_container_bounds_;
-
-  // Indicates that this layer will have a scroll property node and that this
-  // layer's bounds correspond to the scroll node's bounds (both |bounds| and
-  // |scroll_container_bounds|).
-  bool scrollable_ : 1;
-
-  bool should_flatten_screen_space_transform_from_property_tree_ : 1;
+  gfx::Size scroll_contents_bounds_;
+  bool scrollable_ : 1 = false;
 
   // Tracks if drawing-related properties have changed since last redraw.
   // TODO(wutao): We want to distinquish the sources of change so that we can
@@ -530,50 +525,42 @@ class CC_EXPORT LayerImpl {
   // |layer_property_changed_from_property_trees_| does not mean the layer is
   // damaged from animation. We need better mechanism to explicitly capture
   // damage from animations. http://crbug.com/755828.
-  bool layer_property_changed_not_from_property_trees_ : 1;
-  bool layer_property_changed_from_property_trees_ : 1;
-  bool may_contain_video_ : 1;
+  bool layer_property_changed_not_from_property_trees_ : 1 = false;
+  bool layer_property_changed_from_property_trees_ : 1 = false;
 
-  bool masks_to_bounds_ : 1;
-  bool contents_opaque_ : 1;
-  bool use_parent_backface_visibility_ : 1;
-  bool should_check_backface_visibility_ : 1;
-  bool draws_content_ : 1;
-  bool contributes_to_drawn_render_surface_ : 1;
+  bool may_contain_video_ : 1 = false;
+  bool contents_opaque_ : 1 = false;
+  bool contents_opaque_for_text_ : 1 = false;
+  bool should_check_backface_visibility_ : 1 = false;
+  bool draws_content_ : 1 = false;
+  bool contributes_to_drawn_render_surface_ : 1 = false;
 
-  // Tracks if this layer should participate in hit testing.
-  bool hit_testable_ : 1;
-  bool is_resized_by_browser_controls_ : 1;
+  bool is_inner_viewport_scroll_layer_ : 1 = false;
 
-  // TODO(bokan): This can likely be removed after blink-gen-property-trees
-  // is shipped. https://crbug.com/836884.
-  static_assert(LAST_VIEWPORT_LAYER_TYPE < (1u << 3),
-                "enough bits for ViewportLayerType (viewport_layer_type_)");
-  uint8_t viewport_layer_type_ : 3;  // ViewportLayerType
-
-  Region non_fast_scrollable_region_;
+  HitTestOpaqueness hit_test_opaqueness_ = HitTestOpaqueness::kTransparent;
   TouchActionRegion touch_action_region_;
-  Region wheel_event_handler_region_;
-  SkColor background_color_;
-  SkColor safe_opaque_background_color_;
+
+  SkColor4f background_color_ = SkColors::kTransparent;
+  SkColor4f safe_opaque_background_color_ = SkColors::kTransparent;
 
   int transform_tree_index_;
   int effect_tree_index_;
   int clip_tree_index_;
   int scroll_tree_index_;
 
+  std::unique_ptr<RareProperties> rare_properties_;
+
  protected:
   friend class TreeSynchronizer;
 
   DrawMode current_draw_mode_;
   EffectTree& GetEffectTree() const;
-
- private:
   PropertyTrees* GetPropertyTrees() const;
   ClipTree& GetClipTree() const;
   ScrollTree& GetScrollTree() const;
   TransformTree& GetTransformTree() const;
 
+ private:
   ElementId element_id_;
   // Rect indicating what was repainted/updated during update.
   // Note that plugin layers bypass this and leave it empty.
@@ -585,34 +572,27 @@ class CC_EXPORT LayerImpl {
   DrawProperties draw_properties_;
   PerformanceProperties<LayerImpl> performance_properties_;
 
-  std::unique_ptr<base::trace_event::TracedValue> owned_debug_info_;
-  base::trace_event::TracedValue* debug_info_;
+  std::unique_ptr<LayerDebugInfo> debug_info_;
 
   // Cache of all regions represented by any touch action from
   // |touch_action_region_|.
   mutable std::unique_ptr<Region> all_touch_action_regions_;
 
-  bool has_will_change_transform_hint_ : 1;
-  bool needs_push_properties_ : 1;
-  bool is_scrollbar_ : 1;
-  bool scrollbars_hidden_ : 1;
+  bool needs_push_properties_ : 1 = false;
 
-  // The needs_show_scrollbars_ bit tracks a pending request from Blink to show
-  // the overlay scrollbars. It's set on the scroll layer (not the scrollbar
-  // layers) and consumed by LayerTreeImpl::PushPropertiesTo during activation.
-  bool needs_show_scrollbars_ : 1;
+  // The needs_show_scrollbars_ bit tracks a pending request to show the overlay
+  // scrollbars. It's set by UpdateScrollable() on the scroll layer (not the
+  // scrollbar layers) and consumed by LayerTreeImpl::PushPropertiesTo() and
+  // LayerTreeImpl::HandleScrollbarShowRequests().
+  bool needs_show_scrollbars_ : 1 = false;
 
   // This is set for layers that have a property because of which they are not
   // drawn (singular transforms), but they can become visible soon (the property
   // is being animated). For this reason, while these layers are not drawn, they
   // are still rasterized.
-  bool raster_even_if_not_drawn_ : 1;
+  bool raster_even_if_not_drawn_ : 1 = false;
 
-  bool has_transform_node_ : 1;
-
-  // Number of layers mirroring this layer. If greater than zero, forces a
-  // render pass for the layer so it can be embedded by the mirroring layer.
-  int mirror_count_;
+  bool has_transform_node_ : 1 = false;
 };
 
 }  // namespace cc
