@@ -39,10 +39,12 @@
 #include "cc/trees/tree_synchronizer.h"
 #include "components/viz/common/quads/solid_color_draw_quad.h"
 #include "components/viz/test/test_context_provider.h"
-#include "components/viz/test/test_gles2_interface.h"
+#include "components/viz/test/test_raster_interface.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+using ::testing::Bool;
+
 namespace cc {
 
 class FakeResourceTrackingUIResourceManager : public UIResourceManager {
@@ -102,14 +104,17 @@ class FakeResourceTrackingUIResourceManager : public UIResourceManager {
 class BaseScrollbarLayerTest : public testing::Test {
  public:
   explicit BaseScrollbarLayerTest(
-      LayerTreeSettings::ScrollbarAnimator animator) {
+      LayerTreeSettings::ScrollbarAnimator animator,
+      bool enable_fluent_overlay_scrollbar = false) {
     layer_tree_settings_.single_thread_proxy_scheduler = false;
     layer_tree_settings_.use_zero_copy = true;
     layer_tree_settings_.scrollbar_animator = animator;
     layer_tree_settings_.scrollbar_fade_delay = base::Milliseconds(20);
     layer_tree_settings_.scrollbar_fade_duration = base::Milliseconds(20);
+    layer_tree_settings_.enable_fluent_overlay_scrollbar =
+        enable_fluent_overlay_scrollbar;
 
-    animation_host_ = AnimationHost::CreateForTesting(ThreadInstance::MAIN);
+    animation_host_ = AnimationHost::CreateForTesting(ThreadInstance::kMain);
 
     LayerTreeHost::InitParams params;
     params.client = &fake_client_;
@@ -134,7 +139,8 @@ class BaseScrollbarLayerTest : public testing::Test {
   }
 
  protected:
-  raw_ptr<FakeResourceTrackingUIResourceManager> fake_ui_resource_manager_;
+  raw_ptr<FakeResourceTrackingUIResourceManager, DanglingUntriaged>
+      fake_ui_resource_manager_;
   FakeLayerTreeHostClient fake_client_;
   StubLayerTreeHostSingleThreadClient single_thread_client_;
   TestTaskGraphRunner task_graph_runner_;
@@ -149,10 +155,13 @@ class ScrollbarLayerTest : public BaseScrollbarLayerTest {
       : BaseScrollbarLayerTest(LayerTreeSettings::ANDROID_OVERLAY) {}
 };
 
-class AuraScrollbarLayerTest : public BaseScrollbarLayerTest {
+class AuraScrollbarLayerTest : public BaseScrollbarLayerTest,
+                               public testing::WithParamInterface<bool> {
  public:
-  AuraScrollbarLayerTest()
-      : BaseScrollbarLayerTest(LayerTreeSettings::AURA_OVERLAY) {}
+  explicit AuraScrollbarLayerTest(
+      bool enable_fluent_overlay_scrollbar = GetParam())
+      : BaseScrollbarLayerTest(LayerTreeSettings::AURA_OVERLAY,
+                               enable_fluent_overlay_scrollbar) {}
 };
 
 class FakePaintedOverlayScrollbar : public FakeScrollbar {
@@ -317,7 +326,7 @@ TEST_F(ScrollbarLayerTest, ScrollElementIdPushedAcrossCommit) {
           base::MakeRefCounted<FakeNinePatchScrollbar>());
   painted_overlay_scrollbar_layer->SetScrollElementId(layer_a->element_id());
   scoped_refptr<SolidColorScrollbarLayer> solid_color_scrollbar_layer =
-      SolidColorScrollbarLayer::Create(ScrollbarOrientation::VERTICAL, 1, 1,
+      SolidColorScrollbarLayer::Create(ScrollbarOrientation::kVertical, 1, 1,
                                        false);
   solid_color_scrollbar_layer->SetScrollElementId(layer_a->element_id());
 
@@ -374,7 +383,8 @@ TEST_F(ScrollbarLayerTest, ScrollElementIdPushedAcrossCommit) {
     layer_tree_host_->host_impl()->FinishCommit(*commit_state, unsafe_state);
     completion_event->Signal();
   }
-  layer_tree_host_->CommitComplete({base::TimeTicks(), base::TimeTicks::Now()});
+  layer_tree_host_->CommitComplete(commit_state->source_frame_number,
+                                   {base::TimeTicks(), base::TimeTicks::Now()});
 
   EXPECT_EQ(painted_scrollbar_layer_impl->scroll_element_id_,
             layer_b->element_id());
@@ -569,7 +579,7 @@ TEST_F(ScrollbarLayerTest, ThumbRectForOverlayLeftSideVerticalScrollbar) {
   // Create an overlay left side vertical scrollbar.
   scoped_refptr<FakePaintedScrollbarLayer> scrollbar_layer =
       FakePaintedScrollbarLayer::Create(false, true,
-                                        ScrollbarOrientation::VERTICAL, true,
+                                        ScrollbarOrientation::kVertical, true,
                                         true, root_layer->element_id());
   root_layer->SetScrollable(gfx::Size(20, 50));
   root_layer->SetBounds(gfx::Size(50, 100));
@@ -619,7 +629,7 @@ TEST_F(ScrollbarLayerTest, SolidColorDrawQuads) {
     scoped_refptr<Layer> root = Layer::Create();
     scoped_refptr<Layer> child = Layer::Create();
     scoped_refptr<ScrollbarLayerBase> scrollbar_layer =
-        SolidColorScrollbarLayer::Create(ScrollbarOrientation::HORIZONTAL,
+        SolidColorScrollbarLayer::Create(ScrollbarOrientation::kHorizontal,
                                          kThumbThickness, kTrackStart, false);
     root->AddChild(child);
     root->AddChild(scrollbar_layer);
@@ -689,7 +699,7 @@ TEST_F(ScrollbarLayerTest, LayerDrivenSolidColorDrawQuads) {
   scoped_refptr<Layer> child1 = Layer::Create();
   const bool kIsLeftSideVerticalScrollbar = false;
   scoped_refptr<SolidColorScrollbarLayer> child2 =
-      SolidColorScrollbarLayer::Create(ScrollbarOrientation::HORIZONTAL,
+      SolidColorScrollbarLayer::Create(ScrollbarOrientation::kHorizontal,
                                        kThumbThickness, kTrackStart,
                                        kIsLeftSideVerticalScrollbar);
   child2->SetScrollElementId(scroll_layer->element_id());
@@ -745,7 +755,7 @@ TEST_F(ScrollbarLayerTest, ScrollbarLayerOpacity) {
   scoped_refptr<SolidColorScrollbarLayer> scrollbar_layer;
   const bool kIsLeftSideVerticalScrollbar = false;
   scrollbar_layer = SolidColorScrollbarLayer::Create(
-      ScrollbarOrientation::HORIZONTAL, kThumbThickness, kTrackStart,
+      ScrollbarOrientation::kHorizontal, kThumbThickness, kTrackStart,
       kIsLeftSideVerticalScrollbar);
   scrollbar_layer->SetScrollElementId(scroll_layer->element_id());
   scrollbar_layer->SetElementId(ElementId(300));
@@ -807,7 +817,7 @@ TEST_F(ScrollbarLayerTest, ScrollbarLayerOpacity) {
   EXPECT_EQ(node->opacity, 0.25f);
 }
 
-TEST_F(AuraScrollbarLayerTest, ScrollbarLayerPushProperties) {
+TEST_P(AuraScrollbarLayerTest, ScrollbarLayerPushProperties) {
   // Pushing changed bounds of scroll layer can lead to calling
   // OnOpacityAnimated on scrollbar layer which means OnOpacityAnimated should
   // be independent of scrollbar layer's properties as scrollbar layer can push
@@ -821,7 +831,7 @@ TEST_F(AuraScrollbarLayerTest, ScrollbarLayerPushProperties) {
   scoped_refptr<Layer> child1 = Layer::Create();
   const bool kIsLeftSideVerticalScrollbar = false;
   scoped_refptr<SolidColorScrollbarLayer> scrollbar_layer =
-      SolidColorScrollbarLayer::Create(ScrollbarOrientation::HORIZONTAL,
+      SolidColorScrollbarLayer::Create(ScrollbarOrientation::kHorizontal,
                                        kThumbThickness, kTrackStart,
                                        kIsLeftSideVerticalScrollbar);
   scrollbar_layer->SetScrollElementId(scroll_layer->element_id());
@@ -850,7 +860,10 @@ TEST_F(AuraScrollbarLayerTest, ScrollbarLayerPushProperties) {
   const EffectNode* node =
       host_impl->active_tree()->property_trees()->effect_tree().Node(
           scrollbar_layer->effect_tree_index());
-  EXPECT_EQ(node->opacity, 1.f);
+  // If Fluent overlay scrollbars are active, changing the bounds scrollable
+  // content shouldn't make the scrollbars appear.
+  EXPECT_EQ(node->opacity,
+            layer_tree_settings_.enable_fluent_overlay_scrollbar ? 0.f : 1.f);
 }
 
 TEST_F(ScrollbarLayerTest, SubPixelCanScrollOrientation) {
@@ -867,7 +880,7 @@ TEST_F(ScrollbarLayerTest, SubPixelCanScrollOrientation) {
 
   SolidColorScrollbarLayerImpl* scrollbar_layer =
       impl.AddLayer<SolidColorScrollbarLayerImpl>(
-          ScrollbarOrientation::HORIZONTAL, kThumbThickness, kTrackStart,
+          ScrollbarOrientation::kHorizontal, kThumbThickness, kTrackStart,
           kIsLeftSideVerticalScrollbar);
 
   scrollbar_layer->SetScrollElementId(scroll_layer->element_id());
@@ -907,7 +920,7 @@ TEST_F(ScrollbarLayerTest, LayerChangesAffectingScrollbarGeometries) {
   const bool kIsLeftSideVerticalScrollbar = false;
   SolidColorScrollbarLayerImpl* scrollbar_layer =
       impl.AddLayer<SolidColorScrollbarLayerImpl>(
-          ScrollbarOrientation::HORIZONTAL, kThumbThickness, kTrackStart,
+          ScrollbarOrientation::kHorizontal, kThumbThickness, kTrackStart,
           kIsLeftSideVerticalScrollbar);
   scrollbar_layer->SetScrollElementId(scroll_layer->element_id());
   EXPECT_TRUE(impl.host_impl()->active_tree()->ScrollbarGeometriesNeedUpdate());
@@ -983,7 +996,7 @@ TEST_F(ScrollbarLayerTest, UpdateScrollbarGeometriesScrollNodeOnContainer) {
   impl.host_impl()->active_tree()->UpdateScrollbarGeometries();
 }
 
-TEST_F(AuraScrollbarLayerTest, ScrollbarLayerCreateAfterSetScrollable) {
+TEST_P(AuraScrollbarLayerTest, ScrollbarLayerCreateAfterSetScrollable) {
   // Scrollbar Layer can be created after SetScrollable is called and in a
   // separate commit. Ensure we do not missing the DidRequestShowFromMainThread
   // call.
@@ -1010,7 +1023,7 @@ TEST_F(AuraScrollbarLayerTest, ScrollbarLayerCreateAfterSetScrollable) {
   host_impl->ActivateSyncTree();
 
   scoped_refptr<SolidColorScrollbarLayer> scrollbar_layer =
-      SolidColorScrollbarLayer::Create(ScrollbarOrientation::HORIZONTAL,
+      SolidColorScrollbarLayer::Create(ScrollbarOrientation::kHorizontal,
                                        kThumbThickness, kTrackStart,
                                        kIsLeftSideVerticalScrollbar);
   scrollbar_layer->SetScrollElementId(scroll_layer->element_id());
@@ -1041,10 +1054,10 @@ class ScrollbarLayerSolidColorThumbTest : public testing::Test {
     const bool kIsLeftSideVerticalScrollbar = false;
 
     horizontal_scrollbar_layer_ = SolidColorScrollbarLayerImpl::Create(
-        host_impl_->active_tree(), 1, ScrollbarOrientation::HORIZONTAL,
+        host_impl_->active_tree(), 1, ScrollbarOrientation::kHorizontal,
         kThumbThickness, kTrackStart, kIsLeftSideVerticalScrollbar);
     vertical_scrollbar_layer_ = SolidColorScrollbarLayerImpl::Create(
-        host_impl_->active_tree(), 2, ScrollbarOrientation::VERTICAL,
+        host_impl_->active_tree(), 2, ScrollbarOrientation::kVertical,
         kThumbThickness, kTrackStart, kIsLeftSideVerticalScrollbar);
   }
 
@@ -1136,7 +1149,7 @@ class ScrollbarLayerTestResourceCreationAndRelease : public ScrollbarLayerTest {
       const int kTrackStart = 0;
       const bool kIsLeftSideVerticalScrollbar = false;
       scrollbar_layer = SolidColorScrollbarLayer::Create(
-          ScrollbarOrientation::HORIZONTAL, kThumbThickness, kTrackStart,
+          ScrollbarOrientation::kHorizontal, kThumbThickness, kTrackStart,
           kIsLeftSideVerticalScrollbar);
     } else {
       auto scrollbar = base::MakeRefCounted<FakeScrollbar>();
@@ -1420,14 +1433,13 @@ TEST_F(ScaledScrollbarLayerTestResourceCreation, ScaledResourceUpload) {
   // Try something extreme to be larger than max texture size, and make it a
   // non-integer for funsies.
   scoped_refptr<viz::TestContextProvider> context =
-      viz::TestContextProvider::Create();
+      viz::TestContextProvider::CreateRaster();
   // Keep the max texture size reasonable so we don't OOM on low end devices
   // (crbug.com/642333).
-  context->UnboundTestContextGL()->set_max_texture_size(512);
+  constexpr int max_texture_size = 512;
+
+  context->UnboundTestRasterInterface()->set_max_texture_size(max_texture_size);
   context->BindToCurrentSequence();
-  int max_texture_size = 0;
-  context->ContextGL()->GetIntegerv(GL_MAX_TEXTURE_SIZE, &max_texture_size);
-  EXPECT_EQ(512, max_texture_size);
   TestResourceUpload(max_texture_size / 9.9f);
 }
 
@@ -1505,4 +1517,6 @@ TEST_F(ScaledScrollbarLayerTestScaledRasterization, TestLostPrecisionInClip) {
   TestScale(gfx::Rect(0, 1240, 677, 15), 2.46677136f);
 }
 
+INSTANTIATE_TEST_SUITE_P(All, AuraScrollbarLayerTest, Bool());
+
 }  // namespace cc
