@@ -1,12 +1,12 @@
-// Copyright 2014 The Chromium Authors. All rights reserved.
+// Copyright 2014 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #ifndef CC_TREES_SCOPED_ABORT_REMAINING_SWAP_PROMISES_H_
 #define CC_TREES_SCOPED_ABORT_REMAINING_SWAP_PROMISES_H_
 
-#include "base/macros.h"
-#include "cc/output/swap_promise.h"
+#include "base/memory/raw_ptr.h"
+#include "cc/trees/swap_promise.h"
 #include "cc/trees/swap_promise_manager.h"
 
 namespace cc {
@@ -16,15 +16,18 @@ class ScopedAbortRemainingSwapPromises {
   explicit ScopedAbortRemainingSwapPromises(
       SwapPromiseManager* swap_promise_manager)
       : swap_promise_manager_(swap_promise_manager) {}
+  ScopedAbortRemainingSwapPromises(const ScopedAbortRemainingSwapPromises&) =
+      delete;
 
   ~ScopedAbortRemainingSwapPromises() {
     swap_promise_manager_->BreakSwapPromises(SwapPromise::COMMIT_FAILS);
   }
 
- private:
-  SwapPromiseManager* swap_promise_manager_;
+  ScopedAbortRemainingSwapPromises& operator=(
+      const ScopedAbortRemainingSwapPromises&) = delete;
 
-  DISALLOW_COPY_AND_ASSIGN(ScopedAbortRemainingSwapPromises);
+ private:
+  raw_ptr<SwapPromiseManager> swap_promise_manager_;
 };
 
 }  // namespace cc
