@@ -5,6 +5,7 @@
 #include "chrome/browser/extensions/api/passwords_private/password_check_delegate.h"
 
 #include <memory>
+#include <optional>
 #include <string>
 #include <vector>
 
@@ -22,31 +23,30 @@
 #include "base/test/gmock_move_support.h"
 #include "base/test/metrics/histogram_tester.h"
 #include "base/test/mock_callback.h"
-#include "base/test/scoped_feature_list.h"
 #include "base/time/time.h"
 #include "chrome/browser/extensions/api/passwords_private/passwords_private_event_router.h"
 #include "chrome/browser/extensions/api/passwords_private/passwords_private_event_router_factory.h"
+#include "chrome/browser/password_manager/account_password_store_factory.h"
 #include "chrome/browser/password_manager/bulk_leak_check_service_factory.h"
 #include "chrome/browser/password_manager/password_manager_test_util.h"
+#include "chrome/browser/password_manager/profile_password_store_factory.h"
+#include "chrome/browser/signin/identity_manager_factory.h"
+#include "chrome/browser/signin/identity_test_environment_profile_adaptor.h"
 #include "chrome/browser/sync/sync_service_factory.h"
 #include "chrome/common/extensions/api/passwords_private.h"
 #include "chrome/test/base/testing_profile.h"
 #include "components/keyed_service/core/keyed_service.h"
-#include "components/password_manager/content/browser/password_change_success_tracker_factory.h"
 #include "components/password_manager/core/browser/affiliation/fake_affiliation_service.h"
-#include "components/password_manager/core/browser/bulk_leak_check_service.h"
 #include "components/password_manager/core/browser/leak_detection/bulk_leak_check.h"
+#include "components/password_manager/core/browser/leak_detection/bulk_leak_check_service.h"
 #include "components/password_manager/core/browser/leak_detection/leak_detection_delegate_interface.h"
-#include "components/password_manager/core/browser/mock_password_change_success_tracker.h"
-#include "components/password_manager/core/browser/password_change_success_tracker.h"
 #include "components/password_manager/core/browser/password_form.h"
 #include "components/password_manager/core/browser/password_manager_metrics_util.h"
 #include "components/password_manager/core/browser/password_manager_test_utils.h"
-#include "components/password_manager/core/browser/test_password_store.h"
+#include "components/password_manager/core/browser/password_store/test_password_store.h"
 #include "components/password_manager/core/browser/ui/credential_ui_entry.h"
 #include "components/password_manager/core/browser/ui/saved_passwords_presenter.h"
-#include "components/password_manager/core/browser/well_known_change_password_util.h"
-#include "components/password_manager/core/common/password_manager_features.h"
+#include "components/password_manager/core/browser/well_known_change_password/well_known_change_password_util.h"
 #include "components/password_manager/core/common/password_manager_pref_names.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/testing_pref_service.h"
@@ -64,7 +64,6 @@
 #include "services/network/test/test_shared_url_loader_factory.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
-#include "third_party/abseil-cpp/absl/types/optional.h"
 #include "url/origin.h"
 
 namespace extensions {
@@ -99,9 +98,6 @@ using password_manager::InsecurityMetadata;
 using password_manager::IsLeaked;
 using password_manager::IsMuted;
 using password_manager::LeakCheckCredential;
-using password_manager::MockPasswordChangeSuccessTracker;
-using password_manager::PasswordChangeSuccessTracker;
-using password_manager::PasswordChangeSuccessTrackerFactory;
 using password_manager::PasswordForm;
 using password_manager::SavedPasswordsPresenter;
 using password_manager::TestPasswordStore;
@@ -125,61 +121,6 @@ using ::testing::UnorderedElementsAre;
 using MockStartPasswordCheckCallback =
     base::MockCallback<PasswordCheckDelegate::StartPasswordCheckCallback>;
 
-PasswordsPrivateEventRouter* CreateAndUsePasswordsPrivateEventRouter(
-    Profile* profile) {
-  return static_cast<PasswordsPrivateEventRouter*>(
-      PasswordsPrivateEventRouterFactory::GetInstance()
-          ->SetTestingFactoryAndUse(
-              profile,
-              base::BindRepeating([](content::BrowserContext* context) {
-                return std::unique_ptr<KeyedService>(
-                    PasswordsPrivateEventRouter::Create(context));
-              })));
-}
-
-EventRouter* CreateAndUseEventRouter(Profile* profile) {
-  // The factory function only requires that T be a KeyedService. Ensure it is
-  // actually derived from EventRouter to avoid undefined behavior.
-  return static_cast<EventRouter*>(
-      extensions::EventRouterFactory::GetInstance()->SetTestingFactoryAndUse(
-          profile, base::BindRepeating([](content::BrowserContext* context) {
-            return std::unique_ptr<KeyedService>(
-                std::make_unique<EventRouter>(context, nullptr));
-          })));
-}
-
-MockPasswordChangeSuccessTracker* CreateAndUsePasswordChangeSuccessTracker(
-    Profile* profile) {
-  return static_cast<MockPasswordChangeSuccessTracker*>(
-      PasswordChangeSuccessTrackerFactory::GetInstance()
-          ->SetTestingSubclassFactoryAndUse(
-              profile, base::BindRepeating([](content::BrowserContext*) {
-                return std::make_unique<
-                    testing::StrictMock<MockPasswordChangeSuccessTracker>>();
-              })));
-}
-
-BulkLeakCheckService* CreateAndUseBulkLeakCheckService(
-    signin::IdentityManager* identity_manager,
-    Profile* profile) {
-  return static_cast<BulkLeakCheckService*>(
-      BulkLeakCheckServiceFactory::GetInstance()->SetTestingFactoryAndUse(
-          profile, base::BindLambdaForTesting([identity_manager](
-                                                  content::BrowserContext*) {
-            return std::unique_ptr<
-                KeyedService>(std::make_unique<BulkLeakCheckService>(
-                identity_manager,
-                base::MakeRefCounted<network::TestSharedURLLoaderFactory>()));
-          })));
-}
-
-syncer::TestSyncService* CreateAndUseSyncService(Profile* profile) {
-  return SyncServiceFactory::GetInstance()->SetTestingSubclassFactoryAndUse(
-      profile, base::BindLambdaForTesting([](content::BrowserContext*) {
-        return std::make_unique<syncer::TestSyncService>();
-      }));
-}
-
 PasswordForm MakeSavedPassword(
     base::StringPiece signon_realm,
     base::StringPiece16 username,
@@ -248,7 +189,7 @@ auto ExpectCompromisedInfo(
     std::vector<api::passwords_private::CompromiseType> compromise_types) {
   return AllOf(Field(&CompromisedInfo::compromise_time,
                      (base::Time::Now() - elapsed_time_since_compromise)
-                         .ToJsTimeIgnoringNull()),
+                         .InMillisecondsFSinceUnixEpochIgnoringNull()),
                Field(&CompromisedInfo::elapsed_time_since_compromise,
                      elapsed_time_since_compromise_str),
                Field(&CompromisedInfo::compromise_types,
@@ -256,7 +197,7 @@ auto ExpectCompromisedInfo(
 }
 
 // Creates matcher for a given compromised credential
-auto ExpectCredential(const absl::optional<std::string>& change_password_url,
+auto ExpectCredential(const std::optional<std::string>& change_password_url,
                       const std::u16string& username) {
   return AllOf(
       Field(&PasswordUiEntry::username, base::UTF16ToASCII(username)),
@@ -265,7 +206,7 @@ auto ExpectCredential(const absl::optional<std::string>& change_password_url,
 
 // Creates matcher for a given compromised credential
 auto ExpectCompromisedCredential(
-    const absl::optional<std::string>& change_password_url,
+    const std::optional<std::string>& change_password_url,
     const std::u16string& username,
     base::TimeDelta elapsed_time_since_compromise,
     const std::string& elapsed_time_since_compromise_str,
@@ -275,7 +216,7 @@ auto ExpectCompromisedCredential(
           ? Field(&PasswordUiEntry::change_password_url,
                   change_password_url.value())
           : Field(&PasswordUiEntry::change_password_url,
-                  testing::Eq(absl::nullopt));
+                  testing::Eq(std::nullopt));
   return AllOf(
       Field(&PasswordUiEntry::username, base::UTF16ToASCII(username)),
       change_password_url_field_matcher,
@@ -285,6 +226,54 @@ auto ExpectCompromisedCredential(
                                            compromise_types))));
 }
 
+std::unique_ptr<TestingProfile> CreateTestingProfile() {
+  TestingProfile::Builder builder;
+  builder.AddTestingFactory(
+      extensions::EventRouterFactory::GetInstance(),
+      base::BindRepeating([](content::BrowserContext* context)
+                              -> std::unique_ptr<KeyedService> {
+        return std::make_unique<EventRouter>(context, nullptr);
+      }));
+  builder.AddTestingFactory(
+      PasswordsPrivateEventRouterFactory::GetInstance(),
+      base::BindRepeating([](content::BrowserContext* context)
+                              -> std::unique_ptr<KeyedService> {
+        return std::make_unique<PasswordsPrivateEventRouter>(context);
+      }));
+  builder.AddTestingFactory(
+      BulkLeakCheckServiceFactory::GetInstance(),
+      base::BindRepeating([](content::BrowserContext* context)
+                              -> std::unique_ptr<KeyedService> {
+        return std::make_unique<BulkLeakCheckService>(
+            IdentityManagerFactory::GetForProfile(
+                Profile::FromBrowserContext(context)),
+            base::MakeRefCounted<network::TestSharedURLLoaderFactory>());
+      }));
+  builder.AddTestingFactory(
+      ProfilePasswordStoreFactory::GetInstance(),
+      base::BindRepeating(
+          &password_manager::BuildPasswordStore<content::BrowserContext,
+                                                TestPasswordStore>));
+  // SetTestingFactory() can be called always, GetForProfile() will still return
+  // null if account storage is disabled.
+  builder.AddTestingFactory(
+      AccountPasswordStoreFactory::GetInstance(),
+      base::BindRepeating(
+          &password_manager::BuildPasswordStoreWithArgs<
+              content::BrowserContext, password_manager::TestPasswordStore,
+              password_manager::IsAccountStore>,
+          password_manager::IsAccountStore(true)));
+  builder.AddTestingFactory(
+      SyncServiceFactory::GetInstance(),
+      base::BindRepeating(
+          [](content::BrowserContext*) -> std::unique_ptr<KeyedService> {
+            return std::make_unique<syncer::TestSyncService>();
+          }));
+  builder.AddTestingFactories(IdentityTestEnvironmentProfileAdaptor::
+                                  GetIdentityTestEnvironmentFactories());
+  return builder.Build();
+}
+
 class PasswordCheckDelegateTest : public ::testing::Test {
  public:
   PasswordCheckDelegateTest() {
@@ -296,49 +285,57 @@ class PasswordCheckDelegateTest : public ::testing::Test {
   TestEventRouterObserver& event_router_observer() {
     return event_router_observer_;
   }
-  IdentityTestEnvironment& identity_test_env() { return identity_test_env_; }
+  IdentityTestEnvironment& identity_test_env() {
+    return *identity_test_env_profile_adaptor_.identity_test_env();
+  }
   TestingPrefServiceSimple prefs_;
-  TestingProfile& profile() { return profile_; }
-  TestPasswordStore& store() { return *store_; }
-  TestPasswordStore& account_store() { return *account_store_; }
-  BulkLeakCheckService* service() { return bulk_leak_check_service_; }
-  MockPasswordChangeSuccessTracker& password_change_success_tracker() {
-    return *password_change_success_tracker_;
+  TestingProfile& profile() { return *profile_; }
+  TestPasswordStore& store() {
+    return *static_cast<TestPasswordStore*>(
+        ProfilePasswordStoreFactory::GetForProfile(
+            profile_.get(), ServiceAccessType::EXPLICIT_ACCESS)
+            .get());
+  }
+  TestPasswordStore& account_store() {
+    return *static_cast<TestPasswordStore*>(
+        AccountPasswordStoreFactory::GetForProfile(
+            profile_.get(), ServiceAccessType::EXPLICIT_ACCESS)
+            .get());
+  }
+  BulkLeakCheckService* service() {
+    return static_cast<BulkLeakCheckService*>(
+        BulkLeakCheckServiceFactory::GetForProfile(profile_.get()));
+  }
+  syncer::TestSyncService& sync_service() {
+    return *static_cast<syncer::TestSyncService*>(
+        SyncServiceFactory::GetForProfile(profile_.get()));
   }
-  syncer::TestSyncService& sync_service() { return *sync_service_; }
   SavedPasswordsPresenter& presenter() { return presenter_; }
   PasswordCheckDelegate& delegate() { return delegate_; }
 
   PasswordCheckDelegate CreateDelegate(SavedPasswordsPresenter* presenter) {
-    return PasswordCheckDelegate(&profile_, presenter,
+    return PasswordCheckDelegate(profile_.get(), presenter,
                                  &credential_id_generator_);
   }
 
  private:
   content::BrowserTaskEnvironment task_env_{
       base::test::TaskEnvironment::TimeSource::MOCK_TIME};
-  signin::IdentityTestEnvironment identity_test_env_;
-  TestingProfile profile_;
-  raw_ptr<EventRouter> event_router_ = CreateAndUseEventRouter(&profile_);
-  raw_ptr<PasswordsPrivateEventRouter> password_router_ =
-      CreateAndUsePasswordsPrivateEventRouter(&profile_);
-  TestEventRouterObserver event_router_observer_{event_router_};
-  raw_ptr<BulkLeakCheckService> bulk_leak_check_service_ =
-      CreateAndUseBulkLeakCheckService(identity_test_env_.identity_manager(),
-                                       &profile_);
-  scoped_refptr<TestPasswordStore> store_ =
-      CreateAndUseTestPasswordStore(&profile_);
-  scoped_refptr<TestPasswordStore> account_store_ =
-      CreateAndUseTestAccountPasswordStore(&profile_);
-  raw_ptr<MockPasswordChangeSuccessTracker> password_change_success_tracker_ =
-      CreateAndUsePasswordChangeSuccessTracker(&profile_);
-  raw_ptr<syncer::TestSyncService> sync_service_ =
-      CreateAndUseSyncService(&profile_);
+  const std::unique_ptr<TestingProfile> profile_ = CreateTestingProfile();
+  IdentityTestEnvironmentProfileAdaptor identity_test_env_profile_adaptor_{
+      profile_.get()};
+  TestEventRouterObserver event_router_observer_{
+      extensions::EventRouterFactory::GetForBrowserContext(profile_.get())};
   IdGenerator credential_id_generator_;
   password_manager::FakeAffiliationService affiliation_service_;
-  SavedPasswordsPresenter presenter_{&affiliation_service_, store_,
-                                     account_store_};
-  PasswordCheckDelegate delegate_{&profile_, &presenter_,
+  SavedPasswordsPresenter presenter_{&affiliation_service_,
+                                     ProfilePasswordStoreFactory::GetForProfile(
+                                         profile_.get(),
+                                         ServiceAccessType::EXPLICIT_ACCESS),
+                                     AccountPasswordStoreFactory::GetForProfile(
+                                         profile_.get(),
+                                         ServiceAccessType::EXPLICIT_ACCESS)};
+  PasswordCheckDelegate delegate_{profile_.get(), &presenter_,
                                   &credential_id_generator_};
 };
 
@@ -350,7 +347,8 @@ TEST_F(PasswordCheckDelegateTest, GetInsecureCredentialsFillsFieldsCorrectly) {
   store().AddLogin(MakeSavedAndroidPassword(
       kExampleApp, kUsername2, "Example App", kExampleCom, kWeakPassword2));
   RunUntilIdle();
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   RunUntilIdle();
 
   EXPECT_THAT(
@@ -371,7 +369,8 @@ TEST_F(PasswordCheckDelegateTest, WeakCheckNotifiesObservers) {
   EXPECT_FALSE(base::Contains(event_router_observer().events(), kEventName));
 
   // Verify that the event gets fired after weak check is complete.
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   RunUntilIdle();
   EXPECT_EQ(events::PASSWORDS_PRIVATE_ON_INSECURE_CREDENTIALS_CHANGED,
             event_router_observer().events().at(kEventName)->histogram_value);
@@ -381,14 +380,15 @@ TEST_F(PasswordCheckDelegateTest, WeakCheckNotifiesObservers) {
 TEST_F(PasswordCheckDelegateTest, WeakCheckWhenUserSignedOut) {
   store().AddLogin(MakeSavedPassword(kExampleCom, kUsername1, kWeakPassword1));
   RunUntilIdle();
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   RunUntilIdle();
 
   EXPECT_THAT(
       delegate().GetInsecureCredentials(),
       ElementsAre(ExpectCredential(
           "https://example.com/.well-known/change-password", kUsername1)));
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_SIGNED_OUT,
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kSignedOut,
             delegate().GetPasswordCheckStatus().state);
 }
 
@@ -419,23 +419,23 @@ TEST_F(PasswordCheckDelegateTest, GetInsecureCredentialsHandlesTimes) {
       ElementsAre(ExpectCompromisedCredential(
                       "https://example.com/.well-known/change-password",
                       kUsername1, base::Seconds(59), "Just now",
-                      {api::passwords_private::COMPROMISE_TYPE_LEAKED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED}),
+                      {api::passwords_private::CompromiseType::kLeaked,
+                       api::passwords_private::CompromiseType::kReused}),
                   ExpectCompromisedCredential(
                       "https://example.com/.well-known/change-password",
                       kUsername2, base::Seconds(60), "1 minute ago",
-                      {api::passwords_private::COMPROMISE_TYPE_LEAKED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED}),
+                      {api::passwords_private::CompromiseType::kLeaked,
+                       api::passwords_private::CompromiseType::kReused}),
                   ExpectCompromisedCredential(
                       "http://www.example.org/.well-known/change-password",
                       kUsername1, base::Days(100), "3 months ago",
-                      {api::passwords_private::COMPROMISE_TYPE_LEAKED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED}),
+                      {api::passwords_private::CompromiseType::kLeaked,
+                       api::passwords_private::CompromiseType::kReused}),
                   ExpectCompromisedCredential(
                       "http://www.example.org/.well-known/change-password",
                       kUsername2, base::Days(800), "2 years ago",
-                      {api::passwords_private::COMPROMISE_TYPE_LEAKED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED})));
+                      {api::passwords_private::CompromiseType::kLeaked,
+                       api::passwords_private::CompromiseType::kReused})));
 }
 
 // Verifies that both leaked and phished credentials are ordered correctly
@@ -469,25 +469,25 @@ TEST_F(PasswordCheckDelegateTest,
                   ExpectCompromisedCredential(
                       "https://example.com/.well-known/change-password",
                       kUsername1, base::Minutes(1), "1 minute ago",
-                      {api::passwords_private::COMPROMISE_TYPE_LEAKED,
-                       api::passwords_private::COMPROMISE_TYPE_PHISHED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED}),
+                      {api::passwords_private::CompromiseType::kLeaked,
+                       api::passwords_private::CompromiseType::kPhished,
+                       api::passwords_private::CompromiseType::kReused}),
                   ExpectCompromisedCredential(
                       "http://www.example.org/.well-known/change-password",
                       kUsername1, base::Minutes(3), "3 minutes ago",
-                      {api::passwords_private::COMPROMISE_TYPE_PHISHED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED}),
+                      {api::passwords_private::CompromiseType::kPhished,
+                       api::passwords_private::CompromiseType::kReused}),
                   ExpectCompromisedCredential(
                       "http://www.example.org/.well-known/change-password",
                       kUsername2, base::Minutes(4), "4 minutes ago",
-                      {api::passwords_private::COMPROMISE_TYPE_LEAKED,
-                       api::passwords_private::COMPROMISE_TYPE_PHISHED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED}),
+                      {api::passwords_private::CompromiseType::kLeaked,
+                       api::passwords_private::CompromiseType::kPhished,
+                       api::passwords_private::CompromiseType::kReused}),
                   ExpectCompromisedCredential(
                       "https://example.com/.well-known/change-password",
                       kUsername2, base::Minutes(2), "2 minutes ago",
-                      {api::passwords_private::COMPROMISE_TYPE_LEAKED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED})));
+                      {api::passwords_private::CompromiseType::kLeaked,
+                       api::passwords_private::CompromiseType::kReused})));
 }
 
 TEST_F(PasswordCheckDelegateTest, GetInsecureCredentialsInjectsAndroid) {
@@ -514,17 +514,17 @@ TEST_F(PasswordCheckDelegateTest, GetInsecureCredentialsInjectsAndroid) {
                   ExpectCompromisedCredential(
                       "https://example.com/.well-known/change-password",
                       kUsername2, base::Days(3), "3 days ago",
-                      {api::passwords_private::COMPROMISE_TYPE_PHISHED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED}),
+                      {api::passwords_private::CompromiseType::kPhished,
+                       api::passwords_private::CompromiseType::kReused}),
                   ExpectCompromisedCredential(
-                      absl::nullopt, kUsername1, base::Days(4), "4 days ago",
-                      {api::passwords_private::COMPROMISE_TYPE_PHISHED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED}),
+                      std::nullopt, kUsername1, base::Days(4), "4 days ago",
+                      {api::passwords_private::CompromiseType::kPhished,
+                       api::passwords_private::CompromiseType::kReused}),
                   ExpectCompromisedCredential(
                       "https://example.com/.well-known/change-password",
                       kUsername1, base::Minutes(5), "5 minutes ago",
-                      {api::passwords_private::COMPROMISE_TYPE_LEAKED,
-                       api::passwords_private::COMPROMISE_TYPE_REUSED})));
+                      {api::passwords_private::CompromiseType::kLeaked,
+                       api::passwords_private::CompromiseType::kReused})));
 }
 
 // Test that a change to compromised credential notifies observers.
@@ -667,7 +667,8 @@ TEST_F(PasswordCheckDelegateTest, OnLeakFoundDoesNotCreateCredential) {
   PasswordForm form = MakeSavedPassword(kExampleCom, kUsername1, kPassword1);
   store().AddLogin(form);
   RunUntilIdle();
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   store().RemoveLogin(form);
   RunUntilIdle();
   static_cast<BulkLeakCheckDelegateInterface*>(service())->OnFinishedCredential(
@@ -680,10 +681,13 @@ TEST_F(PasswordCheckDelegateTest, OnLeakFoundDoesNotCreateCredential) {
 // Test that we don't create an entry in the password store if IsLeaked is
 // false.
 TEST_F(PasswordCheckDelegateTest, NoLeakedFound) {
+  identity_test_env().MakeAccountAvailable(kTestEmail);
   PasswordForm form = MakeSavedPassword(kExampleCom, kUsername1, kPassword1);
   store().AddLogin(form);
   RunUntilIdle();
 
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   static_cast<BulkLeakCheckDelegateInterface*>(service())->OnFinishedCredential(
       LeakCheckCredential(kUsername1, kPassword1), IsLeaked(false));
   RunUntilIdle();
@@ -700,7 +704,8 @@ TEST_F(PasswordCheckDelegateTest, OnLeakFoundCreatesCredential) {
   store().AddLogin(form);
   RunUntilIdle();
 
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   static_cast<BulkLeakCheckDelegateInterface*>(service())->OnFinishedCredential(
       LeakCheckCredential(kUsername1, kPassword1), IsLeaked(true));
   RunUntilIdle();
@@ -738,7 +743,8 @@ TEST_F(PasswordCheckDelegateTest, OnLeakFoundCreatesMultipleCredential) {
   RunUntilIdle();
 
   identity_test_env().MakeAccountAvailable(kTestEmail);
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   static_cast<BulkLeakCheckDelegateInterface*>(service())->OnFinishedCredential(
       LeakCheckCredential(kUsername1, kPassword1), IsLeaked(true));
   static_cast<BulkLeakCheckDelegateInterface*>(service())->OnFinishedCredential(
@@ -772,7 +778,7 @@ TEST_F(PasswordCheckDelegateTest, OnLeakFoundCreatesMultipleCredential) {
 // Verifies that the case where the user has no saved passwords is reported
 // correctly.
 TEST_F(PasswordCheckDelegateTest, GetPasswordCheckStatusNoPasswords) {
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_NO_PASSWORDS,
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kNoPasswords,
             delegate().GetPasswordCheckStatus().state);
 }
 
@@ -781,7 +787,7 @@ TEST_F(PasswordCheckDelegateTest, GetPasswordCheckStatusIdle) {
   store().AddLogin(MakeSavedPassword(kExampleCom, kUsername1));
   RunUntilIdle();
 
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_IDLE,
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kIdle,
             delegate().GetPasswordCheckStatus().state);
 }
 
@@ -790,8 +796,9 @@ TEST_F(PasswordCheckDelegateTest, GetPasswordCheckStatusSignedOut) {
   store().AddLogin(MakeSavedPassword(kExampleCom, kUsername1));
   RunUntilIdle();
 
-  delegate().StartPasswordCheck();
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_SIGNED_OUT,
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kSignedOut,
             delegate().GetPasswordCheckStatus().state);
 }
 
@@ -802,9 +809,10 @@ TEST_F(PasswordCheckDelegateTest, GetPasswordCheckStatusRunning) {
   store().AddLogin(MakeSavedPassword(kExampleCom, kUsername1));
   RunUntilIdle();
 
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   PasswordCheckStatus status = delegate().GetPasswordCheckStatus();
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_RUNNING, status.state);
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kRunning, status.state);
   EXPECT_EQ(0, *status.already_processed);
   EXPECT_EQ(1, *status.remaining_in_queue);
 
@@ -814,7 +822,7 @@ TEST_F(PasswordCheckDelegateTest, GetPasswordCheckStatusRunning) {
   RunUntilIdle();
 
   status = delegate().GetPasswordCheckStatus();
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_RUNNING, status.state);
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kRunning, status.state);
   EXPECT_EQ(0, *status.already_processed);
   EXPECT_EQ(1, *status.remaining_in_queue);
 }
@@ -828,7 +836,8 @@ TEST_F(PasswordCheckDelegateTest, GetPasswordCheckStatusCount) {
   store().AddLogin(MakeSavedFederatedCredential(kExampleCom, kUsername3));
   RunUntilIdle();
 
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   PasswordCheckStatus status = delegate().GetPasswordCheckStatus();
   EXPECT_EQ(*status.total_number_of_passwords, 2);
 }
@@ -839,11 +848,12 @@ TEST_F(PasswordCheckDelegateTest, GetPasswordCheckStatusOffline) {
   store().AddLogin(MakeSavedPassword(kExampleCom, kUsername1));
   RunUntilIdle();
 
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   identity_test_env().WaitForAccessTokenRequestIfNecessaryAndRespondWithError(
       GoogleServiceAuthError::FromConnectionError(net::ERR_TIMED_OUT));
 
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_OFFLINE,
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kOffline,
             delegate().GetPasswordCheckStatus().state);
 }
 
@@ -854,13 +864,14 @@ TEST_F(PasswordCheckDelegateTest, GetPasswordCheckStatusOther) {
   store().AddLogin(MakeSavedPassword(kExampleCom, kUsername1));
   RunUntilIdle();
 
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   identity_test_env().WaitForAccessTokenRequestIfNecessaryAndRespondWithError(
       GoogleServiceAuthError::FromInvalidGaiaCredentialsReason(
           GoogleServiceAuthError::InvalidGaiaCredentialsReason::
               CREDENTIALS_REJECTED_BY_SERVER));
 
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_SIGNED_OUT,
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kSignedOut,
             delegate().GetPasswordCheckStatus().state);
 }
 
@@ -906,7 +917,8 @@ TEST_F(PasswordCheckDelegateTest,
 
   // Verify that a subsequent call to StartPasswordCheck() results in the
   // expected event.
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   EXPECT_EQ(events::PASSWORDS_PRIVATE_ON_PASSWORD_CHECK_STATUS_CHANGED,
             event_router_observer().events().at(kEventName)->histogram_value);
 }
@@ -923,7 +935,7 @@ TEST_F(PasswordCheckDelegateTest, LastTimePasswordCheckCompletedNotSet) {
 TEST_F(PasswordCheckDelegateTest, LastTimePasswordCheckCompletedIsSet) {
   profile().GetPrefs()->SetDouble(
       kLastTimePasswordCheckCompleted,
-      (base::Time::Now() - base::Minutes(5)).ToDoubleT());
+      (base::Time::Now() - base::Minutes(5)).InSecondsFSinceUnixEpoch());
 
   PasswordCheckStatus status = delegate().GetPasswordCheckStatus();
   EXPECT_THAT(status.elapsed_time_since_last_check,
@@ -933,7 +945,8 @@ TEST_F(PasswordCheckDelegateTest, LastTimePasswordCheckCompletedIsSet) {
 // Checks that a transition into the idle state after starting a check results
 // in resetting the kLastTimePasswordCheckCompleted pref to the current time.
 TEST_F(PasswordCheckDelegateTest, LastTimePasswordCheckCompletedReset) {
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   RunUntilIdle();
 
   service()->set_state_and_notify(BulkLeakCheckService::State::kIdle);
@@ -956,12 +969,13 @@ TEST_F(PasswordCheckDelegateTest, OnCredentialDoneUpdatesProgress) {
   RunUntilIdle();
 
   const auto event_iter = event_router_observer().events().find(kEventName);
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   EXPECT_EQ(events::PASSWORDS_PRIVATE_ON_PASSWORD_CHECK_STATUS_CHANGED,
             event_iter->second->histogram_value);
-  auto status = PasswordCheckStatus::FromValueDeprecated(
-      event_iter->second->event_args.front());
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_RUNNING,
+  auto status =
+      PasswordCheckStatus::FromValue(event_iter->second->event_args.front());
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kRunning,
             status->state);
   EXPECT_EQ(0, *status->already_processed);
   EXPECT_EQ(4, *status->remaining_in_queue);
@@ -969,9 +983,9 @@ TEST_F(PasswordCheckDelegateTest, OnCredentialDoneUpdatesProgress) {
   static_cast<BulkLeakCheckDelegateInterface*>(service())->OnFinishedCredential(
       LeakCheckCredential(kUsername1, kPassword1), IsLeaked(false));
 
-  status = PasswordCheckStatus::FromValueDeprecated(
-      event_iter->second->event_args.front());
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_RUNNING,
+  status =
+      PasswordCheckStatus::FromValue(event_iter->second->event_args.front());
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kRunning,
             status->state);
   EXPECT_EQ(2, *status->already_processed);
   EXPECT_EQ(2, *status->remaining_in_queue);
@@ -979,9 +993,9 @@ TEST_F(PasswordCheckDelegateTest, OnCredentialDoneUpdatesProgress) {
   static_cast<BulkLeakCheckDelegateInterface*>(service())->OnFinishedCredential(
       LeakCheckCredential(kUsername2, kPassword2), IsLeaked(false));
 
-  status = PasswordCheckStatus::FromValueDeprecated(
-      event_iter->second->event_args.front());
-  EXPECT_EQ(api::passwords_private::PASSWORD_CHECK_STATE_RUNNING,
+  status =
+      PasswordCheckStatus::FromValue(event_iter->second->event_args.front());
+  EXPECT_EQ(api::passwords_private::PasswordCheckState::kRunning,
             status->state);
   EXPECT_EQ(4, *status->already_processed);
   EXPECT_EQ(0, *status->remaining_in_queue);
@@ -1006,8 +1020,12 @@ TEST_F(PasswordCheckDelegateTest,
                                         /*account_store=*/nullptr);
   PasswordCheckDelegate delegate = CreateDelegate(&new_presenter);
   new_presenter.Init();
-  delegate.StartPasswordCheck(callback1.Get());
-  delegate.StartPasswordCheck(callback2.Get());
+  delegate.StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck,
+      callback1.Get());
+  delegate.StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck,
+      callback2.Get());
   RunUntilIdle();
 }
 
@@ -1053,7 +1071,8 @@ TEST_F(PasswordCheckDelegateTest,
   store().AddLogin(MakeSavedAndroidPassword(
       kExampleApp, kUsername1, "Example App", kExampleCom, kWeakPassword2));
   RunUntilIdle();
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   RunUntilIdle();
 
   auto credential_with_the_same_password =
@@ -1086,7 +1105,8 @@ TEST_F(PasswordCheckDelegateTest,
   store().AddLogin(MakeSavedPassword(kExampleCom, kUsername1, kWeakPassword1));
   store().AddLogin(MakeSavedPassword(kExampleApp, kUsername2, kWeakPassword1));
   RunUntilIdle();
-  delegate().StartPasswordCheck();
+  delegate().StartPasswordCheck(
+      password_manager::LeakDetectionInitiator::kBulkSyncedPasswordsCheck);
   RunUntilIdle();
 
   EXPECT_EQ(1u, delegate().GetCredentialsWithReusedPassword().size());
@@ -1099,4 +1119,3 @@ TEST_F(PasswordCheckDelegateTest,
 }
 
 }  // namespace extensions
-                          
\ No newline at end of file
