@@ -1,27 +1,39 @@
-// Copyright 2016 The Chromium Authors. All rights reserved.
+// Copyright 2016 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "chrome/browser/media/webrtc/webrtc_text_log_handler.h"
 
 #include <map>
+#include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include "base/bind.h"
+#include "base/check_op.h"
 #include "base/cpu.h"
 #include "base/feature_list.h"
-#include "base/logging.h"
+#include "base/functional/bind.h"
+#include "base/i18n/time_formatting.h"
 #include "base/metrics/histogram_functions.h"
+#include "base/notreached.h"
 #include "base/strings/strcat.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
 #include "base/system/sys_info.h"
-#include "base/task/post_task.h"
+#include "base/task/sequenced_task_runner.h"
 #include "base/time/time.h"
+#include "build/build_config.h"
+#include "build/chromeos_buildflags.h"
+#if !BUILDFLAG(IS_QTWEBENGINE)
+#include "chrome/browser/media/audio_service_util.h"
+#endif
 #include "chrome/common/channel_info.h"
+#if !BUILDFLAG(IS_QTWEBENGINE)
 #include "chrome/common/media/webrtc_logging.mojom.h"
+#else
+#include "qtwebengine/common/media/webrtc_logging.mojom.h"
+#endif
 #include "components/version_info/version_info.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
@@ -32,24 +44,24 @@
 #include "content/public/common/content_features.h"
 #include "gpu/config/gpu_info.h"
 #include "media/audio/audio_manager.h"
-#include "media/webrtc/webrtc_switches.h"
+#include "media/base/media_switches.h"
+#include "media/webrtc/webrtc_features.h"
 #include "net/base/ip_address.h"
 #include "net/base/network_change_notifier.h"
 #include "net/base/network_interfaces.h"
 #include "services/network/public/mojom/network_service.mojom.h"
-#include "services/service_manager/sandbox/features.h"
-#include "services/service_manager/sandbox/sandbox_type.h"
 
-#if defined(OS_LINUX)
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
 #include "base/linux_util.h"
+#include "base/task/thread_pool.h"
 #endif
 
-#if defined(OS_MACOSX)
+#if BUILDFLAG(IS_MAC)
 #include "base/mac/mac_util.h"
 #endif
 
-#if defined(OS_CHROMEOS)
-#include "chromeos/system/statistics_provider.h"
+#if BUILDFLAG(IS_CHROMEOS_ASH)
+#include "chromeos/ash/components/system/statistics_provider.h"
 #endif
 
 using base::NumberToString;
@@ -58,7 +70,7 @@ namespace {
 
 void ForwardMessageViaTaskRunner(
     scoped_refptr<base::SequencedTaskRunner> task_runner,
-    base::Callback<void(const std::string&)> callback,
+    base::RepeatingCallback<void(const std::string&)> callback,
     const std::string& message) {
   task_runner->PostTask(FROM_HERE,
                         base::BindOnce(std::move(callback), message));
@@ -69,8 +81,11 @@ std::string Format(const std::string& message,
                    base::Time start_time) {
   int32_t interval_ms =
       static_cast<int32_t>((timestamp - start_time).InMilliseconds());
-  return base::StringPrintf("[%03d:%03d] %s", interval_ms / 1000,
-                            interval_ms % 1000, message.c_str());
+  // Log start time (current time).
+  const std::string now =
+      base::UnlocalizedTimeFormatWithPattern(base::Time::Now(), "HH:mm:ss.SSS");
+  return base::StringPrintf("[%03d:%03d, %s] %s", interval_ms / 1000,
+                            interval_ms % 1000, now.c_str(), message.c_str());
 }
 
 std::string FormatMetaDataAsLogMessage(const WebRtcLogMetaDataMap& meta_data) {
@@ -110,7 +125,9 @@ std::string IPAddressToSensitiveString(const net::IPAddress& address) {
       sensitive_address = net::IPAddress(stripped).ToString();
       break;
     }
-    default: { break; }
+    default: {
+      break;
+    }
   }
   return sensitive_address;
 #else
@@ -142,17 +159,18 @@ bool WebRtcTextLogHandler::GetChannelIsClosing() const {
 
 void WebRtcTextLogHandler::SetMetaData(
     std::unique_ptr<WebRtcLogMetaDataMap> meta_data,
-    const GenericDoneCallback& callback) {
+    GenericDoneCallback callback) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(!callback.is_null());
 
   if (channel_is_closing_) {
-    FireGenericDoneCallback(callback, false, "The renderer is closing.");
+    FireGenericDoneCallback(std::move(callback), false,
+                            "The renderer is closing.");
     return;
   }
 
   if (logging_state_ != CLOSED && logging_state_ != STARTED) {
-    FireGenericDoneCallback(callback, false,
+    FireGenericDoneCallback(std::move(callback), false,
                             "Meta data must be set before stop or upload.");
     return;
   }
@@ -172,26 +190,28 @@ void WebRtcTextLogHandler::SetMetaData(
       (*meta_data_)[it.first] = it.second;
   }
 
-  FireGenericDoneCallback(callback, true, "");
+  FireGenericDoneCallback(std::move(callback), true, "");
 }
 
-bool WebRtcTextLogHandler::StartLogging(WebRtcLogUploader* log_uploader,
-                                        const GenericDoneCallback& callback) {
+bool WebRtcTextLogHandler::StartLogging(GenericDoneCallback callback) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(!callback.is_null());
 
   if (channel_is_closing_) {
-    FireGenericDoneCallback(callback, false, "The renderer is closing.");
+    FireGenericDoneCallback(std::move(callback), false,
+                            "The renderer is closing.");
     return false;
   }
 
   if (logging_state_ != CLOSED) {
-    FireGenericDoneCallback(callback, false, "A log is already open.");
+    FireGenericDoneCallback(std::move(callback), false,
+                            "A log is already open.");
     return false;
   }
 
-  if (!log_uploader->ApplyForStartLogging()) {
-    FireGenericDoneCallback(callback, false,
+  WebRtcLogUploader* log_uploader = WebRtcLogUploader::GetInstance();
+  if (log_uploader && !log_uploader->ApplyForStartLogging()) {
+    FireGenericDoneCallback(std::move(callback), false,
                             "Cannot start, maybe the maximum number of "
                             "simultaneuos logs has been reached.");
     return false;
@@ -200,9 +220,9 @@ bool WebRtcTextLogHandler::StartLogging(WebRtcLogUploader* log_uploader,
   logging_state_ = STARTING;
 
   DCHECK(!log_buffer_);
-  log_buffer_.reset(new WebRtcLogBuffer());
+  log_buffer_ = std::make_unique<WebRtcLogBuffer>();
   if (!meta_data_)
-    meta_data_.reset(new WebRtcLogMetaDataMap());
+    meta_data_ = std::make_unique<WebRtcLogMetaDataMap>();
 
   content::GetNetworkService()->GetNetworkList(
       net::EXCLUDE_HOST_SCOPE_VIRTUAL_INTERFACES,
@@ -211,12 +231,12 @@ bool WebRtcTextLogHandler::StartLogging(WebRtcLogUploader* log_uploader,
   return true;
 }
 
-void WebRtcTextLogHandler::StartDone(const GenericDoneCallback& callback) {
+void WebRtcTextLogHandler::StartDone(GenericDoneCallback callback) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(!callback.is_null());
 
   if (channel_is_closing_) {
-    FireGenericDoneCallback(callback, false,
+    FireGenericDoneCallback(std::move(callback), false,
                             "Failed to start log. Renderer is closing.");
     return;
   }
@@ -227,29 +247,29 @@ void WebRtcTextLogHandler::StartDone(const GenericDoneCallback& callback) {
 
   logging_started_time_ = base::Time::Now();
   logging_state_ = STARTED;
-  FireGenericDoneCallback(callback, true, "");
+  FireGenericDoneCallback(std::move(callback), true, "");
 }
 
-bool WebRtcTextLogHandler::StopLogging(const GenericDoneCallback& callback) {
+bool WebRtcTextLogHandler::StopLogging(GenericDoneCallback callback) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(!callback.is_null());
 
   if (channel_is_closing_) {
-    FireGenericDoneCallback(callback, false,
+    FireGenericDoneCallback(std::move(callback), false,
                             "Can't stop log. Renderer is closing.");
     return false;
   }
 
   if (logging_state_ != STARTED) {
-    FireGenericDoneCallback(callback, false, "Logging not started.");
+    FireGenericDoneCallback(std::move(callback), false, "Logging not started.");
     return false;
   }
 
-  stop_callback_ = callback;
+  stop_callback_ = std::move(callback);
   logging_state_ = STOPPING;
 
-  base::PostTask(FROM_HERE, {content::BrowserThread::IO},
-                 base::BindOnce(&content::WebRtcLog::ClearLogMessageCallback,
+  content::GetIOThreadTaskRunner({})->PostTask(
+      FROM_HERE, base::BindOnce(&content::WebRtcLog::ClearLogMessageCallback,
                                 render_process_id_));
   return true;
 }
@@ -259,7 +279,7 @@ void WebRtcTextLogHandler::StopDone() {
   DCHECK(stop_callback_);
 
   if (channel_is_closing_) {
-    FireGenericDoneCallback(stop_callback_, false,
+    FireGenericDoneCallback(std::move(stop_callback_), false,
                             "Failed to stop log. Renderer is closing.");
     return;
   }
@@ -272,16 +292,15 @@ void WebRtcTextLogHandler::StopDone() {
   if (logging_state_ == STOPPING) {
     logging_started_time_ = base::Time();
     logging_state_ = STOPPED;
-    FireGenericDoneCallback(stop_callback_, true, "");
-    stop_callback_.Reset();
+    FireGenericDoneCallback(std::move(stop_callback_), true, "");
   }
 }
 
 void WebRtcTextLogHandler::ChannelClosing() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   if (logging_state_ == STARTING || logging_state_ == STARTED) {
-    base::PostTask(FROM_HERE, {content::BrowserThread::IO},
-                   base::BindOnce(&content::WebRtcLog::ClearLogMessageCallback,
+    content::GetIOThreadTaskRunner({})->PostTask(
+        FROM_HERE, base::BindOnce(&content::WebRtcLog::ClearLogMessageCallback,
                                   render_process_id_));
   }
   channel_is_closing_ = true;
@@ -346,10 +365,10 @@ void WebRtcTextLogHandler::LogWebRtcLoggingMessage(
 }
 
 bool WebRtcTextLogHandler::ExpectLoggingStateStopped(
-    const GenericDoneCallback& callback) {
+    GenericDoneCallback* callback) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   if (logging_state_ != STOPPED) {
-    FireGenericDoneCallback(callback, false,
+    FireGenericDoneCallback(std::move(*callback), false,
                             "Logging not stopped or no log open.");
     return false;
   }
@@ -357,7 +376,7 @@ bool WebRtcTextLogHandler::ExpectLoggingStateStopped(
 }
 
 void WebRtcTextLogHandler::FireGenericDoneCallback(
-    const GenericDoneCallback& callback,
+    GenericDoneCallback callback,
     bool success,
     const std::string& error_message) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
@@ -365,8 +384,8 @@ void WebRtcTextLogHandler::FireGenericDoneCallback(
 
   if (error_message.empty()) {
     DCHECK(success);
-    base::SequencedTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE, base::BindOnce(callback, success, error_message));
+    base::SequencedTaskRunner::GetCurrentDefault()->PostTask(
+        FROM_HERE, base::BindOnce(std::move(callback), success, error_message));
     return;
   }
 
@@ -386,16 +405,16 @@ void WebRtcTextLogHandler::FireGenericDoneCallback(
       case LoggingState::STOPPED:
         return "stopped";
     }
-    NOTREACHED();
-    return "";
+    NOTREACHED_NORETURN();
   };
 
   std::string error_message_with_state =
       base::StrCat({error_message, ". State=", state_string(), ". Channel is ",
                     channel_is_closing_ ? "" : "not ", "closing."});
 
-  base::SequencedTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::BindOnce(callback, success, error_message_with_state));
+  base::SequencedTaskRunner::GetCurrentDefault()->PostTask(
+      FROM_HERE,
+      base::BindOnce(std::move(callback), success, error_message_with_state));
 }
 
 void WebRtcTextLogHandler::SetWebAppId(int web_app_id) {
@@ -404,22 +423,34 @@ void WebRtcTextLogHandler::SetWebAppId(int web_app_id) {
 }
 
 void WebRtcTextLogHandler::OnGetNetworkInterfaceList(
-    const GenericDoneCallback& callback,
-    const base::Optional<net::NetworkInterfaceList>& networks) {
+    GenericDoneCallback callback,
+    const std::optional<net::NetworkInterfaceList>& networks) {
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
+  // Hop to a background thread to get the distro string, which can block.
+  base::ThreadPool::PostTaskAndReplyWithResult(
+      FROM_HERE, {base::MayBlock()}, base::BindOnce(&base::GetLinuxDistro),
+      base::BindOnce(&WebRtcTextLogHandler::OnGetNetworkInterfaceListFinish,
+                     weak_factory_.GetWeakPtr(), std::move(callback),
+                     networks));
+#else
+  OnGetNetworkInterfaceListFinish(std::move(callback), networks, "");
+#endif
+}
+
+void WebRtcTextLogHandler::OnGetNetworkInterfaceListFinish(
+    GenericDoneCallback callback,
+    const std::optional<net::NetworkInterfaceList>& networks,
+    const std::string& linux_distro) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
   if (logging_state_ != STARTING || channel_is_closing_) {
-    FireGenericDoneCallback(callback, false, "Logging cancelled.");
+    FireGenericDoneCallback(std::move(callback), false, "Logging cancelled.");
     return;
   }
 
-  // Log start time (current time). We don't use base/i18n/time_formatting.h
-  // here because we don't want the format of the current locale.
-  base::Time::Exploded now = {0};
-  base::Time::Now().LocalExplode(&now);
-  LogToCircularBuffer(base::StringPrintf("Start %d-%02d-%02d %02d:%02d:%02d",
-                                         now.year, now.month, now.day_of_month,
-                                         now.hour, now.minute, now.second));
+  // Log start time (current time).
+  LogToCircularBuffer(base::UnlocalizedTimeFormatWithPattern(
+      base::Time::Now(), "'Start 'y-MM-dd HH:mm:ss"));
 
   // Write metadata if received before logging started.
   if (meta_data_ && !meta_data_->empty()) {
@@ -428,15 +459,19 @@ void WebRtcTextLogHandler::OnGetNetworkInterfaceList(
   }
 
   // Chrome version
-  LogToCircularBuffer("Chrome version: " + version_info::GetVersionNumber() +
-                      " " + chrome::GetChannelName());
+  LogToCircularBuffer(
+      base::StrCat({"Chrome version: ", version_info::GetVersionNumber(), " "
+#if !BUILDFLAG(IS_QTWEBENGINE)
+                    , chrome::GetChannelName(chrome::WithExtendedStable(true))
+#endif
+                   }));
 
   // OS
   LogToCircularBuffer(base::SysInfo::OperatingSystemName() + " " +
                       base::SysInfo::OperatingSystemVersion() + " " +
                       base::SysInfo::OperatingSystemArchitecture());
-#if defined(OS_LINUX)
-  LogToCircularBuffer("Linux distribution: " + base::GetLinuxDistro());
+#if BUILDFLAG(IS_LINUX) || BUILDFLAG(IS_CHROMEOS)
+  { LogToCircularBuffer("Linux distribution: " + linux_distro); }
 #endif
 
   // CPU
@@ -450,11 +485,14 @@ void WebRtcTextLogHandler::OnGetNetworkInterfaceList(
 
   // Computer model
   std::string computer_model = "Not available";
-#if defined(OS_MACOSX)
-  computer_model = base::mac::GetModelIdentifier();
-#elif defined(OS_CHROMEOS)
-  chromeos::system::StatisticsProvider::GetInstance()->GetMachineStatistic(
-      chromeos::system::kHardwareClassKey, &computer_model);
+#if BUILDFLAG(IS_MAC)
+  computer_model = base::SysInfo::HardwareModelName();
+#elif BUILDFLAG(IS_CHROMEOS_ASH)
+  if (const std::optional<base::StringPiece> computer_model_statistic =
+          ash::system::StatisticsProvider::GetInstance()->GetMachineStatistic(
+              ash::system::kHardwareClassKey)) {
+    computer_model = std::string(computer_model_statistic.value());
+  }
 #endif
   LogToCircularBuffer("Computer model: " + computer_model);
 
@@ -476,21 +514,46 @@ void WebRtcTextLogHandler::OnGetNetworkInterfaceList(
   auto enabled_or_disabled_feature_string = [](auto& feature) {
     return base::FeatureList::IsEnabled(feature) ? "enabled" : "disabled";
   };
+#if !BUILDFLAG(IS_QTWEBENGINE)
   auto enabled_or_disabled_bool_string = [](bool value) {
     return value ? "enabled" : "disabled";
   };
+#endif
   LogToCircularBuffer(base::StrCat(
       {"AudioService: OutOfProcess=",
        enabled_or_disabled_feature_string(features::kAudioServiceOutOfProcess),
        ", LaunchOnStartup=",
        enabled_or_disabled_feature_string(
            features::kAudioServiceLaunchOnStartup),
+#if !BUILDFLAG(IS_QTWEBENGINE)
        ", Sandbox=",
-       enabled_or_disabled_bool_string(
-           service_manager::IsAudioSandboxEnabled()),
-       ", ApmInAudioService=",
-       enabled_or_disabled_bool_string(
-           media::IsWebRtcApmInAudioServiceEnabled())}));
+       enabled_or_disabled_bool_string(IsAudioServiceSandboxEnabled())}));
+#else
+       ""}));
+#endif  // !BUILDFLAG(IS_QTWEBENGINE)
+
+#if BUILDFLAG(CHROME_WIDE_ECHO_CANCELLATION)
+  if (media::IsChromeWideEchoCancellationEnabled()) {
+    LogToCircularBuffer(base::StrCat(
+        {"ChromeWideEchoCancellation : Enabled", ", minimize_resampling = ",
+         media::kChromeWideEchoCancellationMinimizeResampling.Get() ? "true"
+                                                                    : "false",
+         ", allow_all_sample_rates = ",
+         media::kChromeWideEchoCancellationAllowAllSampleRates.Get()
+             ? "true"
+             : "false"}));
+  } else {
+    LogToCircularBuffer("ChromeWideEchoCancellation : Disabled");
+  }
+
+  if (base::FeatureList::IsEnabled(media::kDecreaseProcessingAudioFifoSize)) {
+    LogToCircularBuffer(base::StrCat(
+        {"DecreaseProcessingAudioFifoSize : Enabled", ", fifo_size = ",
+         base::NumberToString(media::GetProcessingAudioFifoSize())}));
+  } else {
+    LogToCircularBuffer("DecreaseProcessingAudioFifoSize : Disabled");
+  }
+#endif
 
   // Audio manager
   // On some platforms, this can vary depending on build flags and failure
@@ -516,7 +579,7 @@ void WebRtcTextLogHandler::OnGetNetworkInterfaceList(
         net::NetworkChangeNotifier::ConnectionTypeToString(network.type));
   }
 
-  StartDone(callback);
+  StartDone(std::move(callback));
 
   // After the above data has been written, tell the browser to enable logging.
   // TODO(terelius): Once we have moved over to Mojo, we could tell the
@@ -526,12 +589,13 @@ void WebRtcTextLogHandler::OnGetNetworkInterfaceList(
 
   // TODO(darin): Change SetLogMessageCallback to run on the UI thread.
 
-  auto log_message_callback = base::BindRepeating(
-      &ForwardMessageViaTaskRunner, base::SequencedTaskRunnerHandle::Get(),
-      base::Bind(&WebRtcTextLogHandler::LogMessage,
-                 weak_factory_.GetWeakPtr()));
-  base::PostTask(
-      FROM_HERE, {content::BrowserThread::IO},
+  auto log_message_callback =
+      base::BindRepeating(&ForwardMessageViaTaskRunner,
+                          base::SequencedTaskRunner::GetCurrentDefault(),
+                          base::BindRepeating(&WebRtcTextLogHandler::LogMessage,
+                                              weak_factory_.GetWeakPtr()));
+  content::GetIOThreadTaskRunner({})->PostTask(
+      FROM_HERE,
       base::BindOnce(&content::WebRtcLog::SetLogMessageCallback,
                      render_process_id_, std::move(log_message_callback)));
 }
