@@ -1,4 +1,4 @@
-// Copyright 2019 The Chromium Authors. All rights reserved.
+// Copyright 2019 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,17 +7,16 @@
 #include <string>
 #include <utility>
 
-#include "base/task/post_task.h"
-#include "base/win/windows_version.h"
+#include "base/strings/string_util.h"
+#include "build/branding_buildflags.h"
 #include "chrome/browser/win/conflicts/module_database.h"
 #include "content/public/browser/browser_task_traits.h"
 #include "content/public/browser/browser_thread.h"
 
-#if defined(GOOGLE_CHROME_BUILD)
-#include "base/enterprise_util.h"
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 #include "base/win/win_util.h"
 #include "chrome/browser/win/conflicts/incompatible_applications_updater.h"
-#include "chrome/browser/win/conflicts/module_blacklist_cache_updater.h"
+#include "chrome/browser/win/conflicts/module_blocklist_cache_updater.h"
 #endif
 
 namespace {
@@ -42,7 +41,7 @@ std::string GetProcessTypesString(const ModuleInfoData& module_data) {
   return result;
 }
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
 // Strings used twice.
 constexpr char kNotLoaded[] = "Not loaded";
@@ -50,7 +49,7 @@ constexpr char kAllowedInProcessType[] = "Allowed - Loaded in allowed process";
 constexpr char kAllowedInputMethodEditor[] = "Allowed - Input method editor";
 constexpr char kAllowedMatchingCertificate[] = "Allowed - Matching certificate";
 constexpr char kAllowedMicrosoftModule[] = "Allowed - Microsoft module";
-constexpr char kAllowedWhitelisted[] = "Allowed - Whitelisted";
+constexpr char kAllowedAllowlisted[] = "Allowed - Allowlisted";
 constexpr char kNotAnalyzed[] =
     "Tolerated - Not analyzed (See https://crbug.com/892294)";
 constexpr char kAllowedSameDirectory[] =
@@ -68,25 +67,25 @@ constexpr char kAllowedSameDirectory[] =
 void AppendString(base::StringPiece input, std::string* output) {
   if (!output->empty())
     *output += ", ";
-  input.AppendToString(output);
+  output->append(input);
 }
 
 // Returns a string describing the current module blocking status: loaded or
-// not, blocked or not, was in blacklist cache or not, bypassed blocking or not.
+// not, blocked or not, was in blocklist cache or not, bypassed blocking or not.
 std::string GetBlockingStatusString(
-    const ModuleBlacklistCacheUpdater::ModuleBlockingState& blocking_state) {
+    const ModuleBlocklistCacheUpdater::ModuleBlockingState& blocking_state) {
   std::string status;
 
-  // Output status regarding the blacklist cache, current blocking, and
+  // Output status regarding the blocklist cache, current blocking, and
   // load status.
   if (blocking_state.was_blocked)
     status = "Blocked";
   if (!blocking_state.was_loaded)
     AppendString(kNotLoaded, &status);
-  else if (blocking_state.was_in_blacklist_cache)
+  else if (blocking_state.was_in_blocklist_cache)
     AppendString("Bypassed blocking", &status);
-  if (blocking_state.was_in_blacklist_cache)
-    AppendString("In blacklist cache", &status);
+  if (blocking_state.was_in_blocklist_cache)
+    AppendString("In blocklist cache", &status);
 
   return status;
 }
@@ -95,9 +94,9 @@ std::string GetBlockingStatusString(
 // returns the empty string to indicate that the warning decision description
 // should be used instead.
 std::string GetBlockingDecisionString(
-    const ModuleBlacklistCacheUpdater::ModuleBlockingState& blocking_state,
+    const ModuleBlocklistCacheUpdater::ModuleBlockingState& blocking_state,
     IncompatibleApplicationsUpdater* incompatible_applications_updater) {
-  using BlockingDecision = ModuleBlacklistCacheUpdater::ModuleBlockingDecision;
+  using BlockingDecision = ModuleBlocklistCacheUpdater::ModuleBlockingDecision;
 
   // Append status regarding the logic that will be applied during the next
   // startup.
@@ -117,8 +116,8 @@ std::string GetBlockingDecisionString(
       return kAllowedSameDirectory;
     case BlockingDecision::kAllowedMicrosoft:
       return kAllowedMicrosoftModule;
-    case BlockingDecision::kAllowedWhitelisted:
-      return kAllowedWhitelisted;
+    case BlockingDecision::kAllowedAllowlisted:
+      return kAllowedAllowlisted;
     case BlockingDecision::kNotAnalyzed:
       return kNotAnalyzed;
     case BlockingDecision::kTolerated:
@@ -129,9 +128,9 @@ std::string GetBlockingDecisionString(
         break;
       return "Tolerated - Will be blocked in the future";
     case BlockingDecision::kDisallowedExplicit:
-      return "Disallowed - Explicitly blacklisted";
+      return "Disallowed - Explicitly blocklisted";
     case BlockingDecision::kDisallowedImplicit:
-      return "Disallowed - Implicitly blacklisted";
+      return "Disallowed - Implicitly blocklisted";
   }
 
   // Returning an empty string indicates that the warning status should be used.
@@ -164,15 +163,15 @@ std::string GetModuleWarningDecisionString(
       return kAllowedSameDirectory;
     case WarningDecision::kAllowedMicrosoft:
       return kAllowedMicrosoftModule;
-    case WarningDecision::kAllowedWhitelisted:
-      return kAllowedWhitelisted;
+    case WarningDecision::kAllowedAllowlisted:
+      return kAllowedAllowlisted;
     case WarningDecision::kNotAnalyzed:
       return kNotAnalyzed;
     case WarningDecision::kNoTiedApplication:
       return "Tolerated - Could not tie to an installed application";
     case WarningDecision::kIncompatible:
       return "Incompatible";
-    case WarningDecision::kAddedToBlacklist:
+    case WarningDecision::kAddedToBlocklist:
     case WarningDecision::kUnknown:
       NOTREACHED();
       break;
@@ -184,16 +183,16 @@ std::string GetModuleWarningDecisionString(
 std::string GetModuleStatusString(
     const ModuleInfoKey& module_key,
     IncompatibleApplicationsUpdater* incompatible_applications_updater,
-    ModuleBlacklistCacheUpdater* module_blacklist_cache_updater) {
-  if (!incompatible_applications_updater && !module_blacklist_cache_updater)
+    ModuleBlocklistCacheUpdater* module_blocklist_cache_updater) {
+  if (!incompatible_applications_updater && !module_blocklist_cache_updater)
     return std::string();
 
   std::string status;
 
   // The blocking status is shown over the warning status.
-  if (module_blacklist_cache_updater) {
-    const ModuleBlacklistCacheUpdater::ModuleBlockingState& blocking_state =
-        module_blacklist_cache_updater->GetModuleBlockingState(module_key);
+  if (module_blocklist_cache_updater) {
+    const ModuleBlocklistCacheUpdater::ModuleBlockingState& blocking_state =
+        module_blocklist_cache_updater->GetModuleBlockingState(module_key);
 
     status = GetBlockingStatusString(blocking_state);
 
@@ -216,19 +215,11 @@ std::string GetModuleStatusString(
 
   return status;
 }
-#endif  // defined(GOOGLE_CHROME_BUILD)
+#endif  // BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
 enum ThirdPartyFeaturesStatus {
   // The third-party features are not available in non-Google Chrome builds.
   kNonGoogleChromeBuild,
-  // The third-party features are not available on Windows 7.
-  kNotAvailableWin7,
-  // The third-party features are temporarily disabled on domain-joined
-  // machines because of a known issue with third-party blocking and the
-  // IAttachmentExecute::Save() API (https://crbug.com/870998).
-  // TODO(pmonette): Move IAttachmentExecute::Save() to a utility process and
-  //                 remove this.
-  kEnterpriseManaged,
   // The ThirdPartyBlockingEnabled group policy is disabled.
   kPolicyDisabled,
   // Both the IncompatibleApplicationsWarning and the
@@ -247,9 +238,9 @@ enum ThirdPartyFeaturesStatus {
   kWarningAndBlockingInitialized,
 };
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 ThirdPartyFeaturesStatus GetThirdPartyFeaturesStatus(
-    base::Optional<ThirdPartyConflictsManager::State>
+    std::optional<ThirdPartyConflictsManager::State>
         third_party_conflicts_manager_state) {
   // The ThirdPartyConflictsManager instance exists if we have its state.
   if (third_party_conflicts_manager_state.has_value()) {
@@ -271,21 +262,14 @@ ThirdPartyFeaturesStatus GetThirdPartyFeaturesStatus(
     }
   }
 
-  // Figure out why the manager instance doesn't exist.
-  if (base::win::GetVersion() <= base::win::Version::WIN7)
-    return kNotAvailableWin7;
-
   if (!ModuleDatabase::IsThirdPartyBlockingPolicyEnabled())
     return kPolicyDisabled;
 
   if (!IncompatibleApplicationsUpdater::IsWarningEnabled() &&
-      !ModuleBlacklistCacheUpdater::IsBlockingEnabled()) {
+      !ModuleBlocklistCacheUpdater::IsBlockingEnabled()) {
     return kFeatureDisabled;
   }
 
-  if (base::IsMachineExternallyManaged())
-    return kEnterpriseManaged;
-
   // The above 3 cases are the only possible reasons why the manager wouldn't
   // exist.
   NOTREACHED();
@@ -303,17 +287,9 @@ std::string GetThirdPartyFeaturesStatusString(ThirdPartyFeaturesStatus status) {
     case ThirdPartyFeaturesStatus::kNonGoogleChromeBuild:
       return "The third-party features are not available in non-Google Chrome "
              "builds.";
-    case ThirdPartyFeaturesStatus::kNotAvailableWin7:
-      return "The third-party features are not available on Windows 7.";
-    case ThirdPartyFeaturesStatus::kEnterpriseManaged:
-      return "The third-party features are temporarily disabled for clients on "
-             "domain-joined machines.";
     case ThirdPartyFeaturesStatus::kPolicyDisabled:
       return "The ThirdPartyBlockingEnabled group policy is disabled.";
     case ThirdPartyFeaturesStatus::kFeatureDisabled:
-      if (base::win::GetVersion() < base::win::Version::WIN10)
-        return "The ThirdPartyModulesBlocking feature is disabled.";
-
       return "Both the IncompatibleApplicationsWarning and "
              "ThirdPartyModulesBlocking features are disabled.";
     case ThirdPartyFeaturesStatus::kModuleListInvalid:
@@ -321,17 +297,12 @@ std::string GetThirdPartyFeaturesStatusString(ThirdPartyFeaturesStatus status) {
     case ThirdPartyFeaturesStatus::kNoModuleListAvailable:
       return "Disabled - There is no Module List version available.";
     case ThirdPartyFeaturesStatus::kWarningInitialized:
-      DCHECK_GE(base::win::GetVersion(), base::win::Version::WIN10);
       return "The IncompatibleApplicationsWarning feature is enabled, while "
              "the ThirdPartyModulesBlocking feature is disabled.";
     case ThirdPartyFeaturesStatus::kBlockingInitialized:
-      if (base::win::GetVersion() < base::win::Version::WIN10)
-        return "The ThirdPartyModulesBlocking feature is enabled.";
-
       return "The ThirdPartyModulesBlocking feature is enabled, while the "
              "IncompatibleApplicationsWarning feature is disabled.";
     case ThirdPartyFeaturesStatus::kWarningAndBlockingInitialized:
-      DCHECK_GE(base::win::GetVersion(), base::win::Version::WIN10);
       return "Both the IncompatibleApplicationsWarning and "
              "ThirdPartyModulesBlocking features are enabled";
   }
@@ -340,36 +311,35 @@ std::string GetThirdPartyFeaturesStatusString(ThirdPartyFeaturesStatus status) {
 void OnConflictsDataFetched(
     ConflictsDataFetcher::OnConflictsDataFetchedCallback
         on_conflicts_data_fetched_callback,
-    base::DictionaryValue results,
+    base::Value::Dict results,
     ThirdPartyFeaturesStatus third_party_features_status) {
   // Third-party conflicts status.
-  results.SetBoolean("thirdPartyFeatureEnabled",
-                     IsThirdPartyFeatureEnabled(third_party_features_status));
-  results.SetString(
-      "thirdPartyFeatureStatus",
-      GetThirdPartyFeaturesStatusString(third_party_features_status));
+  results.Set("thirdPartyFeatureEnabled",
+              IsThirdPartyFeatureEnabled(third_party_features_status));
+  results.Set("thirdPartyFeatureStatus",
+              GetThirdPartyFeaturesStatusString(third_party_features_status));
 
   std::move(on_conflicts_data_fetched_callback).Run(std::move(results));
 }
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 void OnModuleDataFetched(ConflictsDataFetcher::OnConflictsDataFetchedCallback
                              on_conflicts_data_fetched_callback,
-                         base::DictionaryValue results,
-                         base::Optional<ThirdPartyConflictsManager::State>
+                         base::Value::Dict results,
+                         std::optional<ThirdPartyConflictsManager::State>
                              third_party_conflicts_manager_state) {
   OnConflictsDataFetched(
       std::move(on_conflicts_data_fetched_callback), std::move(results),
       GetThirdPartyFeaturesStatus(third_party_conflicts_manager_state));
 }
-#endif  // defined(GOOGLE_CHROME_BUILD)
+#endif  // BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
 }  // namespace
 
 ConflictsDataFetcher::~ConflictsDataFetcher() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
-  if (module_list_)
+  if (module_list_.has_value())
     ModuleDatabase::GetInstance()->RemoveObserver(this);
 }
 
@@ -385,7 +355,7 @@ ConflictsDataFetcher::ConflictsDataFetcher(
     OnConflictsDataFetchedCallback on_conflicts_data_fetched_callback)
     : on_conflicts_data_fetched_callback_(
           std::move(on_conflicts_data_fetched_callback))
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
       ,
       weak_ptr_factory_(this)
 #endif
@@ -400,7 +370,7 @@ ConflictsDataFetcher::ConflictsDataFetcher(
 }
 
 void ConflictsDataFetcher::InitializeOnModuleDatabaseTaskRunner() {
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
   // If the ThirdPartyConflictsManager instance exists, wait until it is fully
   // initialized before retrieving the list of modules.
   auto* third_party_conflicts_manager =
@@ -416,7 +386,7 @@ void ConflictsDataFetcher::InitializeOnModuleDatabaseTaskRunner() {
   GetListOfModules();
 }
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
 void ConflictsDataFetcher::OnManagerInitializationComplete(
     ThirdPartyConflictsManager::State state) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
@@ -425,17 +395,17 @@ void ConflictsDataFetcher::OnManagerInitializationComplete(
 
   GetListOfModules();
 }
-#endif  // defined(GOOGLE_CHROME_BUILD)
+#endif  // BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
 void ConflictsDataFetcher::GetListOfModules() {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
 
   // The request is handled asynchronously, filling up the |module_list_|,
   // and will callback via OnModuleDatabaseIdle() on completion.
-  module_list_ = std::make_unique<base::ListValue>();
+  module_list_ = base::Value::List();
 
   auto* module_database = ModuleDatabase::GetInstance();
-  module_database->IncreaseInspectionPriority();
+  module_database->StartInspection();
   module_database->AddObserver(this);
 }
 
@@ -444,41 +414,41 @@ void ConflictsDataFetcher::OnNewModuleFound(const ModuleInfoKey& module_key,
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   DCHECK(module_list_);
 
-  auto data = std::make_unique<base::DictionaryValue>();
+  base::Value::Dict data;
 
-  data->SetString("third_party_module_status", std::string());
-#if defined(GOOGLE_CHROME_BUILD)
+  data.Set("third_party_module_status", std::string());
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
   if (ModuleDatabase::GetInstance()->third_party_conflicts_manager()) {
     auto* incompatible_applications_updater =
         ModuleDatabase::GetInstance()
             ->third_party_conflicts_manager()
             ->incompatible_applications_updater();
-    auto* module_blacklist_cache_updater =
+    auto* module_blocklist_cache_updater =
         ModuleDatabase::GetInstance()
             ->third_party_conflicts_manager()
-            ->module_blacklist_cache_updater();
+            ->module_blocklist_cache_updater();
 
-    data->SetString(
+    data.Set(
         "third_party_module_status",
         GetModuleStatusString(module_key, incompatible_applications_updater,
-                              module_blacklist_cache_updater));
+                              module_blocklist_cache_updater));
   }
-#endif  // defined(GOOGLE_CHROME_BUILD)
+#endif  // BUILDFLAG(GOOGLE_CHROME_BRANDING)
 
   std::string type_string;
   if (module_data.module_properties & ModuleInfoData::kPropertyShellExtension)
     type_string = "Shell extension";
-  data->SetString("type_description", type_string);
+  data.Set("type_description", type_string);
 
   const auto& inspection_result = *module_data.inspection_result;
-  data->SetString("location", inspection_result.location);
-  data->SetString("name", inspection_result.basename);
-  data->SetString("product_name", inspection_result.product_name);
-  data->SetString("description", inspection_result.description);
-  data->SetString("version", inspection_result.version);
-  data->SetString("digital_signer", inspection_result.certificate_info.subject);
-  data->SetString("code_id", GenerateCodeId(module_key));
-  data->SetString("process_types", GetProcessTypesString(module_data));
+  data.Set("location", inspection_result.location);
+  data.Set("name", inspection_result.basename);
+  data.Set("product_name", inspection_result.product_name);
+  data.Set("description", inspection_result.description);
+  data.Set("version", inspection_result.version);
+  data.Set("digital_signer", inspection_result.certificate_info.subject);
+  data.Set("code_id", GenerateCodeId(module_key));
+  data.Set("process_types", GetProcessTypesString(module_data));
 
   module_list_->Append(std::move(data));
 }
@@ -489,23 +459,23 @@ void ConflictsDataFetcher::OnModuleDatabaseIdle() {
 
   ModuleDatabase::GetInstance()->RemoveObserver(this);
 
-  base::DictionaryValue results;
-  results.SetInteger("moduleCount", module_list_->GetSize());
-  results.Set("moduleList", std::move(module_list_));
+  base::Value::Dict results;
+  results.Set("moduleCount", static_cast<int>(module_list_->size()));
+  results.Set("moduleList", std::move(*module_list_));
+  module_list_ = std::nullopt;
 
-#if defined(GOOGLE_CHROME_BUILD)
+#if BUILDFLAG(GOOGLE_CHROME_BRANDING)
   // The state of third-party features must be determined on the UI thread.
-  base::PostTaskWithTraits(
-      FROM_HERE, {content::BrowserThread::UI},
+  content::GetUIThreadTaskRunner({})->PostTask(
+      FROM_HERE,
       base::BindOnce(
           OnModuleDataFetched, std::move(on_conflicts_data_fetched_callback_),
           std::move(results), std::move(third_party_conflicts_manager_state_)));
 #else
   // The third-party features are always disabled on Chromium builds.
-  base::PostTaskWithTraits(
-      FROM_HERE, {content::BrowserThread::UI},
-      base::BindOnce(OnConflictsDataFetched,
-                     std::move(on_conflicts_data_fetched_callback_),
-                     std::move(results), kNonGoogleChromeBuild));
+  content::GetUIThreadTaskRunner({})->PostTask(
+      FROM_HERE, base::BindOnce(OnConflictsDataFetched,
+                                std::move(on_conflicts_data_fetched_callback_),
+                                std::move(results), kNonGoogleChromeBuild));
 #endif
 }
